//
// In order to convert some functionality to Visual C#, the Java Language Conversion Assistant
// creates "support classes" that duplicate the original functionality.  
//
// Support classes replicate the functionality of the original code, but in some cases they are 
// substantially different architecturally. Although every effort is made to preserve the 
// original architecture of the application in the converted project, the user should be aware that 
// the primary goal of these support classes is to replicate functionality, and that at times 
// the architecture of the resulting solution may differ somewhat.
//

using System;

/// <summary>
/// Contains conversion support elements such as classes, interfaces and static methods.
/// </summary>
public class SupportClass
{
	/// <summary>
	/// Writes the exception stack trace to the received stream
	/// </summary>
	/// <param name="throwable">Exception to obtain information from</param>
	/// <param name="stream">Output sream used to write to</param>
	public static void WriteStackTrace(System.Exception throwable, System.IO.TextWriter stream)
	{
		stream.Write(throwable.StackTrace);
		stream.Flush();
	}

	/*******************************/
	/// <summary>
	/// Adds an element to the top end of a Stack instance.
	/// </summary>
	/// <param name="stack">The Stack instance</param>
	/// <param name="element">The element to add</param>
	/// <returns>The element added</returns>  
	public static System.Object StackPush(System.Collections.Stack stack, System.Object element)
	{
		stack.Push(element);
		return element;
	}

	/*******************************/
	/// <summary>
	/// This method returns the literal value received
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static long Identity(long literal)
	{
		return literal;
	}

	/// <summary>
	/// This method returns the literal value received
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static ulong Identity(ulong literal)
	{
		return literal;
	}

	/// <summary>
	/// This method returns the literal value received
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static float Identity(float literal)
	{
		return literal;
	}

	/// <summary>
	/// This method returns the literal value received
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static double Identity(double literal)
	{
		return literal;
	}

	/*******************************/
	/// <summary>
	/// This class contains different methods to manage Collections.
	/// </summary>
	public class CollectionSupport : System.Collections.CollectionBase
	{
		/// <summary>
		/// Creates an instance of the Collection by using an inherited constructor.
		/// </summary>
		public CollectionSupport() : base()
		{			
		}

		/// <summary>
		/// Adds an specified element to the collection.
		/// </summary>
		/// <param name="element">The element to be added.</param>
		/// <returns>Returns true if the element was successfuly added. Otherwise returns false.</returns>
		public virtual bool Add(System.Object element)
		{
			return (this.List.Add(element) != -1);
		}	

		/// <summary>
		/// Adds all the elements contained in the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(System.Collections.ICollection collection)
		{
			bool result = false;
			if (collection!=null)
			{
				System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(collection).GetEnumerator();
				while (tempEnumerator.MoveNext())
				{
					if (tempEnumerator.Current != null)
						result = this.Add(tempEnumerator.Current);
				}
			}
			return result;
		}


		/// <summary>
		/// Adds all the elements contained in the specified support class collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(CollectionSupport collection)
		{
			return this.AddAll((System.Collections.ICollection)collection);
		}

		/// <summary>
		/// Verifies if the specified element is contained into the collection. 
		/// </summary>
		/// <param name="element"> The element that will be verified.</param>
		/// <returns>Returns true if the element is contained in the collection. Otherwise returns false.</returns>
		public virtual bool Contains(System.Object element)
		{
			return this.List.Contains(element);
		}

		/// <summary>
		/// Verifies if all the elements of the specified collection are contained into the current collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be verified.</param>
		/// <returns>Returns true if all the elements are contained in the collection. Otherwise returns false.</returns>
		public virtual bool ContainsAll(System.Collections.ICollection collection)
		{
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(collection).GetEnumerator();
			while (tempEnumerator.MoveNext())
				if (!(result = this.Contains(tempEnumerator.Current)))
					break;
			return result;
		}

		/// <summary>
		/// Verifies if all the elements of the specified collection are contained into the current collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be verified.</param>
		/// <returns>Returns true if all the elements are contained in the collection. Otherwise returns false.</returns>
		public virtual bool ContainsAll(CollectionSupport collection)
		{
			return this.ContainsAll((System.Collections.ICollection) collection);
		}

		/// <summary>
		/// Verifies if the collection is empty.
		/// </summary>
		/// <returns>Returns true if the collection is empty. Otherwise returns false.</returns>
		public virtual bool IsEmpty()
		{
			return (this.Count == 0);
		}

		/// <summary>
		/// Removes an specified element from the collection.
		/// </summary>
		/// <param name="element">The element to be removed.</param>
		/// <returns>Returns true if the element was successfuly removed. Otherwise returns false.</returns>
		public virtual bool Remove(System.Object element)
		{
			bool result = false;
			if (this.Contains(element))
			{
				this.List.Remove(element);
				result = true;
			}
			return result;
		}

		/// <summary>
		/// Removes all the elements contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be removed.</param>
		/// <returns>Returns true if all the elements were successfuly removed. Otherwise returns false.</returns>
		public virtual bool RemoveAll(System.Collections.ICollection collection)
		{ 
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(collection).GetEnumerator();
			while (tempEnumerator.MoveNext())
			{
				if (this.Contains(tempEnumerator.Current))
					result = this.Remove(tempEnumerator.Current);
			}
			return result;
		}

		/// <summary>
		/// Removes all the elements contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be removed.</param>
		/// <returns>Returns true if all the elements were successfuly removed. Otherwise returns false.</returns>
		public virtual bool RemoveAll(CollectionSupport collection)
		{ 
			return this.RemoveAll((System.Collections.ICollection) collection);
		}

		/// <summary>
		/// Removes all the elements that aren't contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to verify the elements that will be retained.</param>
		/// <returns>Returns true if all the elements were successfully removed. Otherwise returns false.</returns>
		public virtual bool RetainAll(System.Collections.ICollection collection)
		{
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			CollectionSupport tempCollection = new CollectionSupport();
			tempCollection.AddAll(collection);
			while (tempEnumerator.MoveNext())
				if (!tempCollection.Contains(tempEnumerator.Current))
				{
					result = this.Remove(tempEnumerator.Current);
					
					if (result == true)
					{
						tempEnumerator = this.GetEnumerator();
					}
				}
			return result;
		}

		/// <summary>
		/// Removes all the elements that aren't contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to verify the elements that will be retained.</param>
		/// <returns>Returns true if all the elements were successfully removed. Otherwise returns false.</returns>
		public virtual bool RetainAll(CollectionSupport collection)
		{
			return this.RetainAll((System.Collections.ICollection) collection);
		}

		/// <summary>
		/// Obtains an array containing all the elements of the collection.
		/// </summary>
		/// <returns>The array containing all the elements of the collection</returns>
		public virtual System.Object[] ToArray()
		{	
			int index = 0;
			System.Object[] objects = new System.Object[this.Count];
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			while (tempEnumerator.MoveNext())
				objects[index++] = tempEnumerator.Current;
			return objects;
		}

		/// <summary>
		/// Obtains an array containing all the elements of the collection.
		/// </summary>
		/// <param name="objects">The array into which the elements of the collection will be stored.</param>
		/// <returns>The array containing all the elements of the collection.</returns>
		public virtual System.Object[] ToArray(System.Object[] objects)
		{	
			int index = 0;
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			while (tempEnumerator.MoveNext())
				objects[index++] = tempEnumerator.Current;
			return objects;
		}

		/// <summary>
		/// Creates a CollectionSupport object with the contents specified in array.
		/// </summary>
		/// <param name="array">The array containing the elements used to populate the new CollectionSupport object.</param>
		/// <returns>A CollectionSupport object populated with the contents of array.</returns>
		public static CollectionSupport ToCollectionSupport(System.Object[] array)
		{
			CollectionSupport tempCollectionSupport = new CollectionSupport();             
			tempCollectionSupport.AddAll(array);
			return tempCollectionSupport;
		}
	}

	/*******************************/
	/// <summary>
	/// This class contains different methods to manage list collections.
	/// </summary>
	public class ListCollectionSupport : System.Collections.ArrayList
	{
		/// <summary>
		/// Creates a new instance of the class ListCollectionSupport.
		/// </summary>
		public ListCollectionSupport() : base()
		{
		}
 
		/// <summary>
		/// Creates a new instance of the class ListCollectionSupport.
		/// </summary>
		/// <param name="collection">The collection to insert into the new object.</param>
		public ListCollectionSupport(System.Collections.ICollection collection) : base(collection)
		{
		}

		/// <summary>
		/// Creates a new instance of the class ListCollectionSupport with the specified capacity.
		/// </summary>
		/// <param name="capacity">The capacity of the new array.</param>
		public ListCollectionSupport(int capacity) : base(capacity)
		{
		}

		/// <summary>
		/// Adds an object to the end of the List.
		/// </summary>          
		/// <param name="valueToInsert">The value to insert in the array list.</param>
		/// <returns>Returns true after adding the value.</returns>
		public virtual bool Add(System.Object valueToInsert)
		{
			base.Insert(this.Count, valueToInsert);
			return true;
		}

		/// <summary>
		/// Adds all the elements contained into the specified collection, starting at the specified position.
		/// </summary>
		/// <param name="index">Position at which to add the first element from the specified collection.</param>
		/// <param name="list">The list used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(int index, System.Collections.IList list)
		{
			bool result = false;
			if (list!=null)
			{
				System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(list).GetEnumerator();
				int tempIndex = index;
				while (tempEnumerator.MoveNext())
				{
					base.Insert(tempIndex++, tempEnumerator.Current);
					result = true;
				}
			}
			return result;
		}

		/// <summary>
		/// Adds all the elements contained in the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(System.Collections.IList collection)
		{
			return this.AddAll(this.Count,collection);
		}

		/// <summary>
		/// Adds all the elements contained in the specified support class collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(CollectionSupport collection)
		{
			return this.AddAll(this.Count,collection);
		}

		/// <summary>
		/// Adds all the elements contained into the specified support class collection, starting at the specified position.
		/// </summary>
		/// <param name="index">Position at which to add the first element from the specified collection.</param>
		/// <param name="list">The list used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(int index, CollectionSupport collection)
		{
			return this.AddAll(index,(System.Collections.IList)collection);
		}
		
		/// <summary>
		/// Creates a copy of the ListCollectionSupport.
		/// </summary>
		/// <returns> A copy of the ListCollectionSupport.</returns>
		public virtual System.Object ListCollectionClone()
		{
			return MemberwiseClone();
		}


		/// <summary>
		/// Returns an iterator of the collection.
		/// </summary>
		/// <returns>An IEnumerator.</returns>
		public virtual System.Collections.IEnumerator ListIterator()
		{
			return base.GetEnumerator();
		}

		/// <summary>
		/// Removes all the elements contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be removed.</param>
		/// <returns>Returns true if all the elements were successfuly removed. Otherwise returns false.</returns>
		public virtual bool RemoveAll(System.Collections.ICollection collection)
		{ 
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(collection).GetEnumerator();
			while (tempEnumerator.MoveNext())
			{
				result = true;
				if (base.Contains(tempEnumerator.Current))
					base.Remove(tempEnumerator.Current);
			}
			return result;
		}
		
		/// <summary>
		/// Removes all the elements contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be removed.</param>
		/// <returns>Returns true if all the elements were successfuly removed. Otherwise returns false.</returns>
		public virtual bool RemoveAll(CollectionSupport collection)
		{ 
			return this.RemoveAll((System.Collections.ICollection) collection);
		}		

		/// <summary>
		/// Removes the value in the specified index from the list.
		/// </summary>          
		/// <param name="index">The index of the value to remove.</param>
		/// <returns>Returns the value removed.</returns>
		public virtual System.Object RemoveElement(int index)
		{
			System.Object objectRemoved = this[index];
			this.RemoveAt(index);
			return objectRemoved;
		}

		/// <summary>
		/// Removes an specified element from the collection.
		/// </summary>
		/// <param name="element">The element to be removed.</param>
		/// <returns>Returns true if the element was successfuly removed. Otherwise returns false.</returns>
		public virtual bool RemoveElement(System.Object element)
		{

			bool result = false;
			if (this.Contains(element))
			{
				base.Remove(element);
				result = true;
			}
			return result;
		}

		/// <summary>
		/// Removes the first value from an array list.
		/// </summary>          
		/// <returns>Returns the value removed.</returns>
		public virtual System.Object RemoveFirst()
		{
			System.Object objectRemoved = this[0];
			this.RemoveAt(0);
			return objectRemoved;
		}

		/// <summary>
		/// Removes the last value from an array list.
		/// </summary>
		/// <returns>Returns the value removed.</returns>
		public virtual System.Object RemoveLast()
		{
			System.Object objectRemoved = this[this.Count-1];
			base.RemoveAt(this.Count-1);
			return objectRemoved;
		}

		/// <summary>
		/// Removes all the elements that aren't contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to verify the elements that will be retained.</param>
		/// <returns>Returns true if all the elements were successfully removed. Otherwise returns false.</returns>
		public virtual bool RetainAll(System.Collections.ICollection collection)
		{
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			ListCollectionSupport tempCollection = new ListCollectionSupport(collection);
			while (tempEnumerator.MoveNext())
				if (!tempCollection.Contains(tempEnumerator.Current))
				{
					result = this.RemoveElement(tempEnumerator.Current);
					
					if (result == true)
					{
						tempEnumerator = this.GetEnumerator();
					}
				}
			return result;
		}
		
		/// <summary>
		/// Removes all the elements that aren't contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to verify the elements that will be retained.</param>
		/// <returns>Returns true if all the elements were successfully removed. Otherwise returns false.</returns>
		public virtual bool RetainAll(CollectionSupport collection)
		{
			return this.RetainAll((System.Collections.ICollection) collection);
		}		

		/// <summary>
		/// Verifies if all the elements of the specified collection are contained into the current collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be verified.</param>
		/// <returns>Returns true if all the elements are contained in the collection. Otherwise returns false.</returns>
		public virtual bool ContainsAll(System.Collections.ICollection collection)
		{
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(collection).GetEnumerator();
			while (tempEnumerator.MoveNext())
				if(!(result = this.Contains(tempEnumerator.Current)))
					break;
			return result;
		}
		
		/// <summary>
		/// Verifies if all the elements of the specified collection are contained into the current collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be verified.</param>
		/// <returns>Returns true if all the elements are contained in the collection. Otherwise returns false.</returns>
		public virtual bool ContainsAll(CollectionSupport collection)
		{
			return this.ContainsAll((System.Collections.ICollection) collection);
		}		

		/// <summary>
		/// Returns a new list containing a portion of the current list between a specified range. 
		/// </summary>
		/// <param name="startIndex">The start index of the range.</param>
		/// <param name="endIndex">The end index of the range.</param>
		/// <returns>A ListCollectionSupport instance containing the specified elements.</returns>
		public virtual ListCollectionSupport SubList(int startIndex, int endIndex)
		{
			int index = 0;
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			ListCollectionSupport result = new ListCollectionSupport();
			for(index = startIndex; index < endIndex; index++)
				result.Add(this[index]);
			return (ListCollectionSupport)result;
		}

		/// <summary>
		/// Obtains an array containing all the elements of the collection.
		/// </summary>
		/// <param name="objects">The array into which the elements of the collection will be stored.</param>
		/// <returns>The array containing all the elements of the collection.</returns>
		public virtual System.Object[] ToArray(System.Object[] objects)
		{	
			if (objects.Length < this.Count)
				objects = new System.Object[this.Count];
			int index = 0;
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			while (tempEnumerator.MoveNext())
				objects[index++] = tempEnumerator.Current;
			return objects;
		}

		/// <summary>
		/// Returns an iterator of the collection starting at the specified position.
		/// </summary>
		/// <param name="index">The position to set the iterator.</param>
		/// <returns>An IEnumerator at the specified position.</returns>
		public virtual System.Collections.IEnumerator ListIterator(int index)
		{
			if ((index < 0) || (index > this.Count)) throw new System.IndexOutOfRangeException();			
			System.Collections.IEnumerator tempEnumerator= this.GetEnumerator();
			if (index > 0)
			{
				int i=0;
				while ((tempEnumerator.MoveNext()) && (i < index - 1))
					i++;
			}
			return tempEnumerator;			
		}
	
		/// <summary>
		/// Gets the last value from a list.
		/// </summary>
		/// <returns>Returns the last element of the list.</returns>
		public virtual System.Object GetLast()
		{
			if (this.Count == 0) throw new System.ArgumentOutOfRangeException();
			else
			{
				return this[this.Count - 1];
			}									 
		}
		
		/// <summary>
		/// Return whether this list is empty.
		/// </summary>
		/// <returns>True if the list is empty, false if it isn't.</returns>
		public virtual bool IsEmpty()
		{
			return (this.Count == 0);
		}
		
		/// <summary>
		/// Replaces the element at the specified position in this list with the specified element.
		/// </summary>
		/// <param name="index">Index of element to replace.</param>
		/// <param name="element">Element to be stored at the specified position.</param>
		/// <returns>The element previously at the specified position.</returns>
		public virtual System.Object Set(int index, System.Object element)
		{
			System.Object result = this[index];
			this[index] = element;
			return result;
		} 

		/// <summary>
		/// Returns the element at the specified position in the list.
		/// </summary>
		/// <param name="index">Index of element to return.</param>
		/// <param name="element">Element to be stored at the specified position.</param>
		/// <returns>The element at the specified position in the list.</returns>
		public virtual System.Object Get(int index)
		{
			return this[index];
		}
	}

	/*******************************/
	/// <summary>
	/// This class manages a set of elements.
	/// </summary>
	public class SetSupport : System.Collections.ArrayList
	{
		/// <summary>
		/// Creates a new set.
		/// </summary>
		public SetSupport(): base()
		{           
		}

		/// <summary>
		/// Creates a new set initialized with System.Collections.ICollection object
		/// </summary>
		/// <param name="collection">System.Collections.ICollection object to initialize the set object</param>
		public SetSupport(System.Collections.ICollection collection): base(collection)
		{           
		}

		/// <summary>
		/// Creates a new set initialized with a specific capacity.
		/// </summary>
		/// <param name="capacity">value to set the capacity of the set object</param>
		public SetSupport(int capacity): base(capacity)
		{           
		}
	 
		/// <summary>
		/// Adds an element to the set.
		/// </summary>
		/// <param name="objectToAdd">The object to be added.</param>
		/// <returns>True if the object was added, false otherwise.</returns>
		public new virtual bool Add(object objectToAdd)
		{
			if (this.Contains(objectToAdd))
				return false;
			else
			{
				base.Add(objectToAdd);
				return true;
			}
		}
	 
		/// <summary>
		/// Adds all the elements contained in the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(System.Collections.ICollection collection)
		{
			bool result = false;
			if (collection!=null)
			{
				System.Collections.IEnumerator tempEnumerator = new System.Collections.ArrayList(collection).GetEnumerator();
				while (tempEnumerator.MoveNext())
				{
					if (tempEnumerator.Current != null)
						result = this.Add(tempEnumerator.Current);
				}
			}
			return result;
		}
		
		/// <summary>
		/// Adds all the elements contained in the specified support class collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be added.</param>
		/// <returns>Returns true if all the elements were successfuly added. Otherwise returns false.</returns>
		public virtual bool AddAll(CollectionSupport collection)
		{
			return this.AddAll((System.Collections.ICollection)collection);
		}
	 
		/// <summary>
		/// Verifies that all the elements of the specified collection are contained into the current collection. 
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be verified.</param>
		/// <returns>True if the collection contains all the given elements.</returns>
		public virtual bool ContainsAll(System.Collections.ICollection collection)
		{
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = collection.GetEnumerator();
			while (tempEnumerator.MoveNext())
				if (!(result = this.Contains(tempEnumerator.Current)))
					break;
			return result;
		}
		
		/// <summary>
		/// Verifies if all the elements of the specified collection are contained into the current collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be verified.</param>
		/// <returns>Returns true if all the elements are contained in the collection. Otherwise returns false.</returns>
		public virtual bool ContainsAll(CollectionSupport collection)
		{
			return this.ContainsAll((System.Collections.ICollection) collection);
		}		
	 
		/// <summary>
		/// Verifies if the collection is empty.
		/// </summary>
		/// <returns>True if the collection is empty, false otherwise.</returns>
		public virtual bool IsEmpty()
		{
			return (this.Count == 0);
		}
	 	 
		/// <summary>
		/// Removes an element from the set.
		/// </summary>
		/// <param name="elementToRemove">The element to be removed.</param>
		/// <returns>True if the element was removed.</returns>
		public new virtual bool Remove(object elementToRemove)
		{
			bool result = false;
			if (this.Contains(elementToRemove))
				result = true;
			base.Remove(elementToRemove);
			return result;
		}
		
		/// <summary>
		/// Removes all the elements contained in the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be removed.</param>
		/// <returns>True if all the elements were successfuly removed, false otherwise.</returns>
		public virtual bool RemoveAll(System.Collections.ICollection collection)
		{ 
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = collection.GetEnumerator();
			while (tempEnumerator.MoveNext())
			{
				if ((result == false) && (this.Contains(tempEnumerator.Current)))
					result = true;
				this.Remove(tempEnumerator.Current);
			}
			return result;
		}
		
		/// <summary>
		/// Removes all the elements contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to extract the elements that will be removed.</param>
		/// <returns>Returns true if all the elements were successfuly removed. Otherwise returns false.</returns>
		public virtual bool RemoveAll(CollectionSupport collection)
		{ 
			return this.RemoveAll((System.Collections.ICollection) collection);
		}		

		/// <summary>
		/// Removes all the elements that aren't contained in the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to verify the elements that will be retained.</param>
		/// <returns>True if all the elements were successfully removed, false otherwise.</returns>
		public virtual bool RetainAll(System.Collections.ICollection collection)
		{
			bool result = false;
			System.Collections.IEnumerator tempEnumerator = collection.GetEnumerator();
			SetSupport tempSet = (SetSupport)collection;
			while (tempEnumerator.MoveNext())
				if (!tempSet.Contains(tempEnumerator.Current))
				{
					result = this.Remove(tempEnumerator.Current);
					tempEnumerator = this.GetEnumerator();
				}
			return result;
		}
		
		/// <summary>
		/// Removes all the elements that aren't contained into the specified collection.
		/// </summary>
		/// <param name="collection">The collection used to verify the elements that will be retained.</param>
		/// <returns>Returns true if all the elements were successfully removed. Otherwise returns false.</returns>
		public virtual bool RetainAll(CollectionSupport collection)
		{
			return this.RetainAll((System.Collections.ICollection) collection);
		}		
	 
		/// <summary>
		/// Obtains an array containing all the elements of the collection.
		/// </summary>
		/// <returns>The array containing all the elements of the collection.</returns>
		public new virtual object[] ToArray()
		{
			int index = 0;
			object[] tempObject= new object[this.Count];
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			while (tempEnumerator.MoveNext())
				tempObject[index++] = tempEnumerator.Current;
			return tempObject;
		}

		/// <summary>
		/// Obtains an array containing all the elements in the collection.
		/// </summary>
		/// <param name="objects">The array into which the elements of the collection will be stored.</param>
		/// <returns>The array containing all the elements of the collection.</returns>
		public virtual object[] ToArray(object[] objects)
		{
			int index = 0;
			System.Collections.IEnumerator tempEnumerator = this.GetEnumerator();
			while (tempEnumerator.MoveNext())
				objects[index++] = tempEnumerator.Current;
			return objects;
		}
	}
	/*******************************/
	/// <summary>
	/// This class manages different operation with collections.
	/// </summary>
	public class AbstractSetSupport : SetSupport
	{
		/// <summary>
		/// The constructor with no parameters to create an abstract set.
		/// </summary>
		public AbstractSetSupport()
		{
		}
	}


	/*******************************/
	/// <summary> 
	/// This class manages a hash set of elements.
	/// </summary>
	public class HashSetSupport : AbstractSetSupport
	{
		/// <summary>
		/// Creates a new hash set collection.
		/// </summary>
		public HashSetSupport()
		{     
		}
	       
		/// <summary>
		/// Creates a new hash set collection.
		/// </summary>
		/// <param name="collection">The collection to initialize the hash set with.</param>
		public HashSetSupport(System.Collections.ICollection collection)
		{
			this.AddRange(collection);
		}
	       
		/// <summary>
		/// Creates a new hash set with the given capacity.
		/// </summary>
		/// <param name="capacity">The initial capacity of the hash set.</param>
		public HashSetSupport(int capacity)
		{
			this.Capacity = capacity;
		}
	    
		/// <summary>
		/// Creates a new hash set with the given capacity.
		/// </summary>
		/// <param name="capacity">The initial capacity of the hash set.</param>
		/// <param name="loadFactor">The load factor of the hash set.</param>
		public HashSetSupport(int capacity, float loadFactor)
		{
			this.Capacity = capacity;
		}

		/// <summary>
		/// Creates a copy of the HashSetSupport.
		/// </summary>
		/// <returns> A copy of the HashSetSupport.</returns>
		public virtual object HashSetClone()
		{
			return MemberwiseClone();
		}
	}
	/*******************************/
	/// <summary>
	/// Initializes a new instance of the ArrayList class that contains elements copied from the specified collection 
	/// and that has the same initial capacity as the number of elements copied.
	/// </summary>
	/// <param name="collection">The ICollection whose elements are copied to the new list.</param>
	/// <returns>The new instance, if the collection received as parameter was System.Collection.Stack type the new instance 
	/// contains the elements in reverse order.</returns>
	public static ListCollectionSupport CreateCollection(System.Collections.ICollection collection)
	{
		ListCollectionSupport arrayStack = new ListCollectionSupport(collection);
		if (collection is System.Collections.Stack)
			arrayStack.Reverse();
		return arrayStack;
	}


	/*******************************/
	/// <summary>
	/// Give functions to obtain information of graphic elements
	/// </summary>
	public class GraphicsManager
	{
		//Instance of GDI+ drawing surfaces graphics hashtable
		static public GraphicsHashTable manager = new GraphicsHashTable();

		/// <summary>
		/// Creates a new Graphics object from the device context handle associated with the Graphics
		/// parameter
		/// </summary>
		/// <param name="oldGraphics">Graphics instance to obtain the parameter from</param>
		/// <returns>A new GDI+ drawing surface</returns>
		public static System.Drawing.Graphics CreateGraphics(System.Drawing.Graphics oldGraphics)
		{
			System.IntPtr hdc = oldGraphics.GetHdc();
			oldGraphics.ReleaseHdc(hdc);
			return System.Drawing.Graphics.FromHdc(hdc);
		}

		/// <summary>
		/// This method draws a Bezier curve.
		/// </summary>
		/// <param name="graphics">It receives the Graphics instance</param>
		/// <param name="array">An array of (x,y) pairs of coordinates used to draw the curve.</param>
		public static void Bezier(System.Drawing.Graphics graphics, int[] array)
		{
			System.Drawing.Pen pen;
			pen = GraphicsManager.manager.GetPen(graphics);
			graphics.DrawBezier(pen, array[0], array[1], array[2], array[3], array[4], array[5], array[6], array[7]);
		}

		/// <summary>
		/// Gets the text size width and height from a given GDI+ drawing surface and a given font
		/// </summary>
		/// <param name="graphics">Drawing surface to use</param>
		/// <param name="graphicsFont">Font type to measure</param>
		/// <param name="text">String of text to measure</param>
		/// <returns>A point structure with both size dimentions; x for width and y for height</returns>
		public static System.Drawing.Point GetTextSize(System.Drawing.Graphics graphics, System.Drawing.Font graphicsFont, System.String text)
		{
			System.Drawing.Point textSize;
			System.Drawing.SizeF tempSizeF;
			tempSizeF = graphics.MeasureString(text, graphicsFont);
			textSize = new System.Drawing.Point();
			textSize.X = (int) tempSizeF.Width;
			textSize.Y = (int) tempSizeF.Height;
			return textSize;
		}

		/// <summary>
		/// Gets the text size width and height from a given GDI+ drawing surface and a given font
		/// </summary>
		/// <param name="graphics">Drawing surface to use</param>
		/// <param name="graphicsFont">Font type to measure</param>
		/// <param name="text">String of text to measure</param>
		/// <param name="width">Maximum width of the string</param>
		/// <param name="format">StringFormat object that represents formatting information, such as line spacing, for the string</param>
		/// <returns>A point structure with both size dimentions; x for width and y for height</returns>
		public static System.Drawing.Point GetTextSize(System.Drawing.Graphics graphics, System.Drawing.Font graphicsFont, System.String text, System.Int32 width, System.Drawing.StringFormat format)
		{
			System.Drawing.Point textSize;
			System.Drawing.SizeF tempSizeF;
			tempSizeF = graphics.MeasureString(text, graphicsFont, width, format);
			textSize = new System.Drawing.Point();
			textSize.X = (int) tempSizeF.Width;
			textSize.Y = (int) tempSizeF.Height;
			return textSize;
		}


		/// <summary>
		/// Gives functionality over a hashtable of GDI+ drawing surfaces
		/// </summary>
		public class GraphicsHashTable:System.Collections.Hashtable 
		{
			/// <summary>
			/// Gets the graphic color characteristics from a given control
			/// </summary>
			/// <param name="control">Control to obtain the data from</param>
			/// <returns>A graphic object with the control's characteristics</returns>
			public System.Drawing.Graphics GetGraphics(System.Windows.Forms.Control control)
			{
				System.Drawing.Graphics graphic;
				if (control.Visible == true)
				{
					graphic = control.CreateGraphics();
					if (this[graphic] == null)
					{
						GraphicsProperties tempProps = new GraphicsProperties();
						tempProps.color = control.ForeColor;
						tempProps.BackColor = control.BackColor;
						tempProps.TextColor = control.ForeColor;
						tempProps.GraphicFont = control.Font;
						Add(graphic, tempProps);
					}
				}
				else
				{
					graphic = null;
				}
				return graphic;
			}

			/// <summary>
			/// Sets the background color property to the given graphics object in the hashtable. If the element doesn't exist, then it adds the graphic element to the hashtable with the given background color.
			/// </summary>
			/// <param name="graphic">Graphic element to search or add</param>
			/// <param name="color">Background color to set</param>
			public void SetBackColor(System.Drawing.Graphics graphic, System.Drawing.Color color)
			{
				if (this[graphic] != null)
					((GraphicsProperties) this[graphic]).BackColor = color;
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.BackColor = color;
					Add(graphic, tempProps);
				}
			}

			/// <summary>
			/// Gets the background color property to the given graphics object in the hashtable. If the element doesn't exist, then it returns White.
			/// </summary>
			/// <param name="graphic">Graphic element to search</param>
			/// <returns>The background color of the graphic</returns>
			public System.Drawing.Color GetBackColor(System.Drawing.Graphics graphic)
			{
				if (this[graphic] == null)
					return System.Drawing.Color.White;
				else
					return ((GraphicsProperties) this[graphic]).BackColor;
			}

			/// <summary>
			/// Sets the text color property to the given graphics object in the hashtable. If the element doesn't exist, then it adds the graphic element to the hashtable with the given text color.
			/// </summary>
			/// <param name="graphic">Graphic element to search or add</param>
			/// <param name="color">Text color to set</param>
			public void SetTextColor(System.Drawing.Graphics graphic, System.Drawing.Color color)
			{
				if (this[graphic] != null)
					((GraphicsProperties) this[graphic]).TextColor = color;
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.TextColor = color;
					Add(graphic, tempProps);
				}
			}

			/// <summary>
			/// Gets the text color property to the given graphics object in the hashtable. If the element doesn't exist, then it returns White.
			/// </summary>
			/// <param name="graphic">Graphic element to search</param>
			/// <returns>The text color of the graphic</returns>
			public System.Drawing.Color GetTextColor(System.Drawing.Graphics graphic) 
			{
				if (this[graphic] == null)
					return System.Drawing.Color.White;
				else
					return ((GraphicsProperties) this[graphic]).TextColor;
			}

			/// <summary>
			/// Sets the GraphicBrush property to the given graphics object in the hashtable. If the element doesn't exist, then it adds the graphic element to the hashtable with the given GraphicBrush.
			/// </summary>
			/// <param name="graphic">Graphic element to search or add</param>
			/// <param name="brush">GraphicBrush to set</param>
			public void SetBrush(System.Drawing.Graphics graphic, System.Drawing.SolidBrush brush) 
			{
				if (this[graphic] != null)
					((GraphicsProperties) this[graphic]).GraphicBrush = brush;
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.GraphicBrush = brush;
					Add(graphic, tempProps);
				}
			}

			/// <summary>
			/// Gets the SolidBrush property to the given graphics object in the hashtable. If the element doesn't exist, then it returns Black.
			/// </summary>
			/// <param name="graphic">Graphic element to search</param>
			/// <returns>The SolidBrush setting of the graphic</returns>
			public System.Drawing.SolidBrush GetBrush(System.Drawing.Graphics graphic)
			{
				if (this[graphic] == null)
					return new System.Drawing.SolidBrush(System.Drawing.Color.Black);
				else
					return ((GraphicsProperties) this[graphic]).GraphicBrush;
			}

			/// <summary>
			/// Sets the GraphicPen property to the given graphics object in the hashtable. If the element doesn't exist, then it adds the graphic element to the hashtable with the given Pen.
			/// </summary>
			/// <param name="graphic">Graphic element to search or add</param>
			/// <param name="pen">Pen to set</param>
			public void SetPen(System.Drawing.Graphics graphic, System.Drawing.Pen pen) 
			{
				if (this[graphic] != null)
					((GraphicsProperties) this[graphic]).GraphicPen = pen;
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.GraphicPen = pen;
					Add(graphic, tempProps);
				}
			}

			/// <summary>
			/// Gets the GraphicPen property to the given graphics object in the hashtable. If the element doesn't exist, then it returns Black.
			/// </summary>
			/// <param name="graphic">Graphic element to search</param>
			/// <returns>The GraphicPen setting of the graphic</returns>
			public System.Drawing.Pen GetPen(System.Drawing.Graphics graphic)
			{
				if (this[graphic] == null)
					return System.Drawing.Pens.Black;
				else
					return ((GraphicsProperties) this[graphic]).GraphicPen;
			}

			/// <summary>
			/// Sets the GraphicFont property to the given graphics object in the hashtable. If the element doesn't exist, then it adds the graphic element to the hashtable with the given Font.
			/// </summary>
			/// <param name="graphic">Graphic element to search or add</param>
			/// <param name="Font">Font to set</param>
			public void SetFont(System.Drawing.Graphics graphic, System.Drawing.Font font) 
			{
				if (this[graphic] != null)
					((GraphicsProperties) this[graphic]).GraphicFont = font;
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.GraphicFont = font;
					Add(graphic,tempProps);
				}
			}

			/// <summary>
			/// Gets the GraphicFont property to the given graphics object in the hashtable. If the element doesn't exist, then it returns Microsoft Sans Serif with size 8.25.
			/// </summary>
			/// <param name="graphic">Graphic element to search</param>
			/// <returns>The GraphicFont setting of the graphic</returns>
			public System.Drawing.Font GetFont(System.Drawing.Graphics graphic)
			{
				if (this[graphic] == null)
					return new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
				else
					return ((GraphicsProperties) this[graphic]).GraphicFont;
			}

			/// <summary>
			/// Sets the color properties for a given Graphics object. If the element doesn't exist, then it adds the graphic element to the hashtable with the color properties set with the given value.
			/// </summary>
			/// <param name="graphic">Graphic element to search or add</param>
			/// <param name="color">Color value to set</param>
			public void SetColor(System.Drawing.Graphics graphic, System.Drawing.Color color) 
			{
				if (this[graphic] != null)
				{
					((GraphicsProperties) this[graphic]).GraphicPen.Color = color;
					((GraphicsProperties) this[graphic]).GraphicBrush.Color = color;
					((GraphicsProperties) this[graphic]).color = color;
				}
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.GraphicPen.Color = color;
					tempProps.GraphicBrush.Color = color;
					tempProps.color = color;
					Add(graphic,tempProps);
				}
			}

			/// <summary>
			/// Gets the color property to the given graphics object in the hashtable. If the element doesn't exist, then it returns Black.
			/// </summary>
			/// <param name="graphic">Graphic element to search</param>
			/// <returns>The color setting of the graphic</returns>
			public System.Drawing.Color GetColor(System.Drawing.Graphics graphic) 
			{
				if (this[graphic] == null)
					return System.Drawing.Color.Black;
				else
					return ((GraphicsProperties) this[graphic]).color;
			}

			/// <summary>
			/// This method gets the TextBackgroundColor of a Graphics instance
			/// </summary>
			/// <param name="graphic">The graphics instance</param>
			/// <returns>The color value in ARGB encoding</returns>
			public System.Drawing.Color GetTextBackgroundColor(System.Drawing.Graphics graphic)
			{
				if (this[graphic] == null)
					return System.Drawing.Color.Black;
				else 
				{ 
					return ((GraphicsProperties) this[graphic]).TextBackgroundColor;
				}
			}

			/// <summary>
			/// This method set the TextBackgroundColor of a Graphics instace
			/// </summary>
			/// <param name="graphic">The graphics instace</param>
			/// <param name="color">The System.Color to set the TextBackgroundColor</param>
			public void SetTextBackgroundColor(System.Drawing.Graphics graphic, System.Drawing.Color color) 
			{
				if (this[graphic] != null)
				{
					((GraphicsProperties) this[graphic]).TextBackgroundColor = color;								
				}
				else
				{
					GraphicsProperties tempProps = new GraphicsProperties();
					tempProps.TextBackgroundColor = color;				
					Add(graphic,tempProps);
				}
			}

			/// <summary>
			/// Structure to store properties from System.Drawing.Graphics objects
			/// </summary>
			class GraphicsProperties
			{
				public System.Drawing.Color TextBackgroundColor = System.Drawing.Color.Black;
				public System.Drawing.Color color = System.Drawing.Color.Black;
				public System.Drawing.Color BackColor = System.Drawing.Color.White;
				public System.Drawing.Color TextColor = System.Drawing.Color.Black;
				public System.Drawing.SolidBrush GraphicBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
				public System.Drawing.Pen   GraphicPen = new System.Drawing.Pen(System.Drawing.Color.Black);
				public System.Drawing.Font  GraphicFont = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			}
		}
	}
	/*******************************/
	/// <summary>
	/// Determines whether two Collections instances are equals.
	/// </summary>
	/// <param name="source">The first Collections to compare. </param>
	/// <param name="target">The second Collections to compare. </param>
	/// <returns>Return true if the first collection is the same instance as the second collection, otherwise return false.</returns>
	public static bool EqualsSupport(System.Collections.ICollection source, System.Collections.ICollection target )
	{
		System.Collections.IEnumerator sourceEnumerator = ReverseStack(source);
		System.Collections.IEnumerator targetEnumerator = ReverseStack(target);
     
		if (source.Count != target.Count)
			return false;
		while(sourceEnumerator.MoveNext() && targetEnumerator.MoveNext())
			if (!sourceEnumerator.Current.Equals(targetEnumerator.Current))
				return false;
		return true;
	}
	
	/// <summary>
	/// Determines if a Collection is equal to the Object.
	/// </summary>
	/// <param name="source">The first Collections to compare.</param>
	/// <param name="target">The Object to compare.</param>
	/// <returns>Return true if the first collection contains the same values of the second Object, otherwise return false.</returns>
	public static bool EqualsSupport(System.Collections.ICollection source, System.Object target)
	{
		if((target.GetType())!= (typeof(System.Collections.ICollection)))
			return false;
		else
			return EqualsSupport(source,(System.Collections.ICollection)target);
	}

	/// <summary>
	/// Determines if a IDictionaryEnumerator is equal to the Object.
	/// </summary>
	/// <param name="source">The first IDictionaryEnumerator to compare.</param>
	/// <param name="target">The second Object to compare.</param>
	/// <returns>Return true if the first IDictionaryEnumerator contains the same values of the second Object, otherwise return false.</returns>
	public static bool EqualsSupport(System.Collections.IDictionaryEnumerator source, System.Object target)
	{
		if((target.GetType())!= (typeof(System.Collections.IDictionaryEnumerator)))
			return false;
		else
			return EqualsSupport(source,(System.Collections.IDictionaryEnumerator)target);
	}

	/// <summary>
	/// Determines whether two IDictionaryEnumerator instances are equals.
	/// </summary>
	/// <param name="source">The first IDictionaryEnumerator to compare.</param>
	/// <param name="target">The second IDictionaryEnumerator to compare.</param>
	/// <returns>Return true if the first IDictionaryEnumerator contains the same values as the second IDictionaryEnumerator, otherwise return false.</returns>
	public static bool EqualsSupport(System.Collections.IDictionaryEnumerator source, System.Collections.IDictionaryEnumerator target )
	{
		while(source.MoveNext() && target.MoveNext())
			if (source.Key.Equals(target.Key))
				if(source.Value.Equals(target.Value))
					return true;
		return false;
	}

	/// <summary>
	/// Reverses the Stack Collection received.
	/// </summary>
	/// <param name="collection">The collection to reverse.</param>
	/// <returns>The collection received in reverse order if it was a System.Collections.Stack type, otherwise it does 
	/// nothing to the collection.</returns>
	public static System.Collections.IEnumerator ReverseStack(System.Collections.ICollection collection)
	{
		if((collection.GetType()) == (typeof(System.Collections.Stack)))
		{
			System.Collections.ArrayList collectionStack = new System.Collections.ArrayList(collection);
			collectionStack.Reverse();
			return collectionStack.GetEnumerator();
		}
		else
			return collection.GetEnumerator();
	}

	/*******************************/
	//Provides access to a static System.Random class instance
	static public System.Random Random = new System.Random();

	/*******************************/
	/// <summary>
	/// Creates a GraphicsPath from two Int Arrays with a specific number of points.
	/// </summary>
	/// <param name="xPoints">Int Array to set the X points of the GraphicsPath</param>
	/// <param name="yPoints">Int Array to set the Y points of the GraphicsPath</param>
	/// <param name="pointsNumber">Number of points to add to the GraphicsPath</param>
	/// <returns>A new GraphicsPath</returns>
	public static System.Drawing.Drawing2D.GraphicsPath CreateGraphicsPath(int[] xPoints, int[] yPoints, int pointsNumber)
	{
		System.Drawing.Drawing2D.GraphicsPath tempGraphicsPath = new System.Drawing.Drawing2D.GraphicsPath();
		if (pointsNumber == 2)
			tempGraphicsPath.AddLine(xPoints[0], yPoints[0], xPoints[1], yPoints[1]);
		else
		{
			System.Drawing.Point[] tempPointArray = new System.Drawing.Point[pointsNumber];
			for (int index = 0; index < pointsNumber; index++)
				tempPointArray[index] = new System.Drawing.Point(xPoints[index], yPoints[index]);

			tempGraphicsPath.AddPolygon(tempPointArray);
		}
		return tempGraphicsPath;
	}

	/*******************************/
	/// <summary>
	/// Adds a new key-and-value pair into the hash table
	/// </summary>
	/// <param name="collection">The collection to work with</param>
	/// <param name="key">Key used to obtain the value</param>
	/// <param name="newValue">Value asociated with the key</param>
	/// <returns>The old element associated with the key</returns>
	public static System.Object PutElement(System.Collections.IDictionary collection, System.Object key, System.Object newValue)
	{
		System.Object element = collection[key];
		collection[key] = newValue;
		return element;
	}

	/*******************************/
	/// <summary>
	/// Removes the element with the specified key from a Hashtable instance.
	/// </summary>
	/// <param name="hashtable">The Hashtable instance</param>
	/// <param name="key">The key of the element to remove</param>
	/// <returns>The element removed</returns>  
	public static System.Object HashtableRemove(System.Collections.Hashtable hashtable, System.Object key)
	{
		System.Object element = hashtable[key];
		hashtable.Remove(key);
		return element;
	}

	/*******************************/
	/// <summary>
	/// Calculates the descent of the font, using the GetCellDescent and GetEmHeight
	/// </summary>
	/// <param name="font">The Font instance used to obtain the Descent</param>
	/// <returns>The Descent of the font </returns>
	public static int GetDescent(System.Drawing.Font font)
	{		
		System.Drawing.FontFamily fontFamily = font.FontFamily;
		int descent = fontFamily.GetCellDescent(font.Style);
		int descentPixel = (int) font.Size * descent / fontFamily.GetEmHeight(font.Style);
		return descentPixel;
	}

	/*******************************/
	/// <summary>
	/// This class contains static methods to manage arrays.
	/// </summary>
	public class ArrayListSupport
	{
		/// <summary>
		/// Obtains an array containing all the elements of the collection. 
		/// </summary>
		/// <param name="collection">The collection from wich to obtain the elements.</param>
		/// <param name="objects">The array containing all the elements of the collection.</param>
		/// <returns>The array containing all the elements of the collection.</returns>
		public static System.Object[] ToArray(System.Collections.ArrayList collection, System.Object[] objects)
		{	
			int index = 0;
			System.Collections.IEnumerator tempEnumerator = collection.GetEnumerator();
			while (tempEnumerator.MoveNext())
				objects[index++] = tempEnumerator.Current;
			return objects;
		}
	}


}
