using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The AbstractPotential class is a class that implements most of the methods
	/// of the Potential interface. In particular it supports all the methods that
	/// do not relate to the way data is stored.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public abstract class AbstractPotential : Potential
	{
		public abstract double[] Data{get;set;}
		virtual public int IdenticalValues
		{
			set
			{
				double[] data = new double[size];
				for (int i = 0; i < size; i++)
					data[i] = value;
				this.Data = data;
			}
			
		}
		virtual public NodeList Nodes
		{
			get
			{
				return nodes;
			}
			
		}
		virtual public int Size
		{
			get
			{
				return size;
			}
			
		}
		virtual public int ParentStateCount
		{
			get
			{
				return this.parentStateCount;
			}
			
		}
		virtual public int StateCount
		{
			get
			{
				return this.stateCount;
			}
			
		}
		/// <summary> The list of nodes defining the configurations of this table.</summary>
		protected internal NodeList nodes;
		
		/// <summary> An array of state counts of the nodes.</summary>
		protected internal int[] maximums;
		
		/// <summary> Holds the next configuration to be returned by the iterator.</summary>
		protected internal int[] states;
		
		/// <summary> Whether or not the iterator is currently reset.</summary>
		protected internal bool initialized = false;
		
		/// <summary> The number of states of the node that owns this table.</summary>
		protected internal int stateCount;
		
		/// <summary> The number of parent configurations</summary>
		protected internal int parentStateCount;
		
		/// <summary> The total number of entries of this table.</summary>
		protected internal int size;
		
		
		/// <summary> Defines the fields of this table.</summary>
		/// <param name="nodes">The list of nodes defining the configurations of the table.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		protected internal AbstractPotential(NodeList nodes)
		{
			this.nodes = nodes;
			System.Collections.IEnumerator i = nodes.GetEnumerator();
			int n = 1;
			parentStateCount = 0;
			stateCount = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (i.MoveNext())
			{
				try
				{
					parentStateCount = n;
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode parent = (DiscreteNode) i.Current;
					stateCount = parent.getNumberOfStates();
					n = n * stateCount;
				}
				catch (System.InvalidCastException e)
				{
					throw new MLPOperationNotAllowedException("The node list supplied contains invalid node type(s)");
				}
			}
			size = stateCount * parentStateCount;
			this.updateMaximums();
		}
		
		internal virtual void  updateMaximums()
		{
			NodeList nodes = this.Nodes;
			System.Collections.IEnumerator ite = nodes.ListIterator();
			int nc = nodes.Count;
			maximums = new int[nc];
			int counter = 0;
			//UPGRADE_TODO: Method 'java.util.ListIterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilListIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.ListIterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilListIteratornext"'
				DiscreteNode node = (DiscreteNode) ite.Current;
				int sc = node.getNumberOfStates();
				maximums[counter] = sc;
				counter++;
			}
		}
		
		public abstract double getDataItem(int parm1);
		
		public abstract double getDataItem(int[] state);
		
		public abstract void  setDataItem(int index, double value_Renamed);
		
		public virtual int convertToIndex(int[] state)
		{
			int index = 0;
			int nc = maximums.Length;
			int factor = 1;
			for (int i = nc - 1; i > - 1; i--)
			{
				index = index + state[i] * factor;
				factor = factor * maximums[i];
			}
			return index;
		}
		
		public virtual int convertToParentIndex(int[] state)
		{
			int index = 0;
			int nc = maximums.Length - 1;
			int factor = 1;
			for (int i = nc - 1; i > - 1; i--)
			{
				index = index + state[i] * factor;
				factor = factor * maximums[i];
			}
			return index;
		}
		
		public abstract void  setDataItem(int[] state, double value_Renamed);
		
		public virtual bool hasNextState()
		{
			if (initialized)
				return true;
			int p = maximums.Length - 1;
			states[p]++;
			while (p > - 1 && states[p] >= maximums[p])
			{
				states[p] = 0;
				if (p > 0)
					states[p - 1]++;
				p--;
			}
			return (p > - 1);
		}
		
		public virtual int[] nextState()
		{
			initialized = false;
			return states;
		}
		
		public virtual void  initialize()
		{
			states = new int[maximums.Length];
			for (int i = 0; i < states.Length; i++)
				states[i] = 0;
			initialized = true;
		}
		
		public virtual PotentialTable contractWith(PotentialTable ut)
		{
			double[] utPotentials = ut.Data;
			double[] contraction = new double[utPotentials.Length];
			PotentialTable cont = new PotentialTable(ut.Nodes);
			for (int i = 0; i < contraction.Length; i++)
				contraction[i] = this.getDataItem(i) * utPotentials[i];
			cont.Data = contraction;
			return cont;
		}
		
		public virtual int[] convertParentIndexToStateArray(int index)
		{
			int[] state = new int[nodes.Count - 1];
			int factor = 1;
			int dif = 0;
			for (int p = state.Length - 1; p > - 1; p--)
			{
				state[p] = ((index - dif) % (maximums[p] * factor) / factor);
				dif = dif + state[p] * factor;
				factor = maximums[p] * factor;
			}
			return state;
		}
		
		private System.String padString(System.String s, int l)
		{
			if (s.Length >= l)
				return s.Substring(0, (l) - (0));
			System.Text.StringBuilder buffer = new System.Text.StringBuilder(s);
			while (buffer.Length < l)
				buffer.Append(" ");
			return buffer.ToString();
		}
		
		public override System.String ToString()
		{
			return toString(null);
		}
		
		public virtual System.String toString(Potential otherTable)
		{
			System.Text.StringBuilder buffer = new System.Text.StringBuilder();
			int[] lengths = new int[nodes.Count];
			
			buffer.Append("+");
			for (int i = 0; i < lengths.Length; i++)
			{
				Node node = (Node) nodes[i];
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				lengths[i] = node.ToString().Length;
				if (lengths[i] < 8)
					lengths[i] = 8;
				for (int j = 0; j < lengths[i]; j++)
					buffer.Append("-");
				buffer.Append("+");
			}
			buffer.Append("----------");
			buffer.Append("+");
			if (otherTable != null)
			{
				buffer.Append("----------");
				buffer.Append("+");
			}
			buffer.Append("\n");
			
			buffer.Append("|");
			for (int i = 0; i < lengths.Length; i++)
			{
				Node node = (Node) nodes[i];
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				buffer.Append(this.padString(node.ToString(), lengths[i]));
				buffer.Append("|");
			}
			buffer.Append("Probabili.");
			buffer.Append("|");
			if (otherTable != null)
			{
				buffer.Append(" Utility  ");
				buffer.Append("|");
			}
			
			buffer.Append("\n");
			
			buffer.Append("+");
			for (int i = 0; i < lengths.Length; i++)
			{
				for (int j = 0; j < lengths[i]; j++)
					buffer.Append("-");
				buffer.Append("+");
			}
			buffer.Append("----------");
			buffer.Append("+");
			if (otherTable != null)
			{
				buffer.Append("----------");
				buffer.Append("+");
			}
			buffer.Append("\n");
			
			this.initialize();
			while (this.hasNextState())
			{
				int[] state = this.nextState();
				buffer.Append("|");
				for (int i = 0; i < lengths.Length; i++)
				{
					DiscreteNode node = (DiscreteNode) nodes[i];
					buffer.Append(this.padString(node.getStateLabel(state[i]), lengths[i]));
					buffer.Append("|");
				}
				buffer.Append(this.padString("" + this.getDataItem(state), 10));
				buffer.Append("|");
				if (otherTable != null)
				{
					buffer.Append(this.padString("" + otherTable.getDataItem(state), 10));
					buffer.Append("|");
				}
				buffer.Append("\n");
			}
			
			buffer.Append("+");
			for (int i = 0; i < lengths.Length; i++)
			{
				for (int j = 0; j < lengths[i]; j++)
					buffer.Append("-");
				buffer.Append("+");
			}
			buffer.Append("----------");
			buffer.Append("+");
			if (otherTable != null)
			{
				buffer.Append("----------");
				buffer.Append("+");
			}
			buffer.Append("\n");
			
			return buffer.ToString();
		}
	}
}