using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The superclass of all decision nodes. Supports all functionality of a
	/// decision node in addition to those of a DiscreteNode.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class DiscreteDecisionChanceNode:DiscreteNode
	{
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the path to the previous decision in the decision node ordering.</summary>
		/// <returns> A list of cliques.
		/// </returns>
		/// <seealso cref="dk.esthauge.limid.api.DiscreteDecisionChanceNode#pathToPrevious">
		/// </seealso>
		/// <summary> Sets the path to the previous decision in the decision node ordering. The
		/// method is called by the API. It should not be called directly.
		/// </summary>
		/// <param name="path">A list of cliques.
		/// </param>
		/// <seealso cref="dk.esthauge.limid.api.DiscreteDecisionChanceNode#pathToPrevious">
		/// </seealso>
		virtual public SupportClass.ListCollectionSupport PathToPrevious
		{
			get
			{
				return pathToPrevious;
			}
			
			set
			{
				pathToPrevious = value;
			}
			
		}
		/// <summary> Returns the string "decision".</summary>
		/// <returns> The string "decision".
		/// </returns>
		override public System.String Type
		{
			get
			{
				return "decision";
			}
			
		}
		/// <summary> Returns the color of a decision node. This default implementation uses
		/// the static field LimidDomain.decisionNodeColor for representation of
		/// this color. Subclasses may override the method for more sofisticated
		/// coloring.
		/// </summary>
		/// <returns> The color.
		/// </returns>
		/// <seealso cref="LimidDomain#decisionNodeColor">
		/// </seealso>
		override protected internal System.Drawing.Color Color
		{
			get
			{
				return this.HomeDomain.decisionNodeColor;
			}
			
		}
		/// <summary> This general decision node class returns the empty string.</summary>
		/// <returns>
		/// </returns>
		override protected internal System.String Subtype
		{
			get
			{
				return "";
			}
			
		}
		//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
		/// <summary> Returns the home clique of this decision.</summary>
		/// <returns> The home clique.
		/// </returns>
		/// <summary> Sets the home clique of this node. The home clique is defined as a clique
		/// containg the family of the node. The method is used by the API. It should
		/// not be called directly.
		/// </summary>
		/// <param name="clique">The home clique.
		/// </param>
		virtual public LimidClique HomeClique
		{
			get
			{
				return this.homeClique;
			}
			
			set
			{
				this.homeClique = value;
			}
			
		}
		
		/// <summary> Used for creating unique default names</summary>
		internal static int counter = 0;
		
		/// <summary> A list of cliques. Identifies the unique path in the junction tree of a
		/// compiled model from the clique of this node to the clique of the previous
		/// node in the decision ordering.
		/// </summary>
		protected internal SupportClass.ListCollectionSupport pathToPrevious;
		
		/// <summary> The home clique of this decision. Identifies the clique containing the
		/// family of this decision node in a compiled model.
		/// </summary>
		protected internal LimidClique homeClique;
		
		/// <summary> Creates a new decision node of the specified domain.</summary>
		/// <param name="ld">The domain of the node.
		/// @throws MLPDuplicateNodeNameException
		/// </param>
		public DiscreteDecisionChanceNode(LimidDomain ld):base(ld)
		{
			counter++;
		}
		
		/// <returns> A list of cliques.
		/// </returns>
		
		/// <summary> The defaultname is the letter "D" followed by a number.</summary>
		protected internal override void  setDefaultName()
		{
			name = "D" + counter;
		}
		
		/// <summary> The method performs Single Policy Updating of the policy for this node.</summary>
		/// <param name="first">A flag indicating whether this is first time the method is
		/// called for this decision. If true, a full propagation is performed.
		/// Otherwise a partial propagation along the path represented by the field
		/// pathToPrevios is performed.
		/// </param>
		/// <returns> The utility value after the updating.
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		/// <seealso cref="dk.esthauge.limid.api.DiscreteDecisionChanceNode#pathToPrevious">
		/// </seealso>
		public virtual double singlePolicyUpdating(bool first)
		{
			
			// Find the home clique of the decision
			LimidClique clique = (LimidClique) HomeClique;
			
			// Force full propagation - debugging only
			// first = true;
			
			// Collect to this clique
			if (first)
				clique.collectToClique(null);
			else
				clique.collectToCliqueAlongPath(this.PathToPrevious);
			
			// Retract current policy, i.e. add all other decisions than this (if any)
			LimidPotential workingCopy = clique.Potential;
			if (clique.DecisionPotentials.Count > 1)
			{
				System.Collections.IEnumerator ite = clique.DecisionPotentials.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Potential table = (Potential) ite.Current;
					if (table != this.Table)
					{
						// This is where the name "workingCopy" makes sense. The method
						// combineCopyWith... returns a NEW potential
						workingCopy = workingCopy.combineCopyWithProbabilities(table);
					}
				}
			}
			
			// Marginalize both tables to the family of this node
			LimidPotential marginal = workingCopy.marginalize(this.Table.Nodes);
			PotentialTable contraction = marginal.Contraction;
			
			// Just for debugging
			//System.out.println(this+", Prior Utility value = "
			//                   +marginal.getExpectedUtility());
			
			// Iterate over parent states
			int thisDec = 0;
			int optimalDecision = 0;
			double max = 0;
			// New policy for this decision
			double[] policy = new double[this.Table.Size];
			// Counter for parent state
			int pc = - 1;
			contraction.initialize();
			while (contraction.hasNextState())
			{
				int[] states = contraction.nextState();
				// Get index of current decision
				thisDec = states[states.Length - 1];
				// Get value of current decision
				double value_Renamed = contraction.getDataItem(states);
				if (thisDec == 0)
				{
					// This is a new parent state
					pc++;
					max = value_Renamed;
					optimalDecision = thisDec;
				}
				else
				{
					// We are still in the old parent state - is this decision better?
					if (value_Renamed > max)
					{
						max = value_Renamed;
						optimalDecision = thisDec;
					}
				}
				// Define decision as optimal if this was the last decision alternative
				int sc = this.getNumberOfStates();
				if (thisDec == sc - 1)
				{
					for (int i = 0; i < sc; i++)
						if (i == optimalDecision)
							policy[i + pc * sc] = 1;
						else
							policy[i + pc * sc] = 0;
				}
			}
			// Update policy. This is enough since it is the original table which is
			// put in the list
			this.Table.Data = policy;
			
			workingCopy = workingCopy.combineCopyWithProbabilities(this.Table);
			
			double expectedUtility = workingCopy.ExpectedUtility;
			
			// Just for debugging
			// System.out.println("Utility value = "+expectedUtility);
			
			return expectedUtility;
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.lang.Object.clone' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		protected internal System.Object Clone()
		{
			try
			{
				DiscreteDecisionChanceNode newNode = new DiscreteDecisionChanceNode(this.HomeDomain);
				newNode.copyNode(this);
				return newNode;
			}
			catch (MLPException ex)
			{
				return null;
			}
		}
	}
}