using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The abstract super class of all discrete nodes (i.e. chance nodes and
	/// decision nodes). Discrete nodes have a number of distinct states.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public abstract class DiscreteNode:Node
	{
		/// <summary> A list of state labels.</summary>
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		protected internal System.Collections.ArrayList stateLabels;
		
		/// <summary> Creates a discrete node within a specified domain model.</summary>
		/// <param name="ld">The domain model where the node is to be creatated.
		/// @throws MLPDuplicateNodeNameException The new node is automatically given
		/// a default name. Since names must be unique within domain model, it may
		/// happen that the user has already manually given the default name of this
		/// node to an other node. In that case, an exception is thrown.
		/// </param>
		public DiscreteNode(LimidDomain ld):base(ld)
		{
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			stateLabels = new System.Collections.ArrayList();
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			stateLabels.Add("State 0");
			createInitialTable();
		}
		
		/// <summary> Creates the probability table of this node. Usually it is NOT necessary
		/// to call this method directly. It is called automatically when needed.
		/// @throws MLPOperationNotAllowedException Thrown if one of the parents is
		/// not a discrete node (typically it is then a utility node)
		/// </summary>
		public override void  createTable()
		{
			NodeList nodes = new NodeList(this.Parents);
			nodes.Add(this);
			this.table = new DefaultProbabilityPotential(nodes);
			//    this.table = new PotentialTable(nodes);
			this.table.IdenticalValues = 1;
		}
		
		/// <summary> Returns the number of states of this discrete node.</summary>
		/// <returns> The number of states.
		/// </returns>
		public virtual int getNumberOfStates()
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			return stateLabels.Count;
		}
		
		/// <summary> Defines a descriptive label for a given state.</summary>
		/// <param name="index">The index of the state.
		/// </param>
		/// <param name="label">The desired label.
		/// </param>
		public virtual void  setStateLabel(int index, System.String label)
		{
			System.Object item;
			item = stateLabels[index];
			stateLabels[index] = label;
			System.Object generatedAux = item;
		}
		
		/// <summary> Returns the descriptive label of a specific state.</summary>
		/// <param name="index">The index of the state.
		/// </param>
		/// <returns> The descriptive label.
		/// </returns>
		public virtual System.String getStateLabel(int index)
		{
			return (System.String) stateLabels[index];
		}
		
		/// <summary> Adjusts the number of states of this discrete node. The contents of the
		/// potential table of this node as well as of all child nodes are lost, since
		/// the tables must be restructured.
		/// </summary>
		/// <param name="number">The desired state count.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  setNumberOfStates(int number)
		{
			if (number < 1)
				throw new System.ArgumentException("The number of states of a discrete node must be at least 1");
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			if (number > stateLabels.Count)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				for (int i = stateLabels.Count; i < number; i++)
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					stateLabels.Add("State " + i);
				}
			}
			else
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				if (number < stateLabels.Count)
				{
					stateLabels.RetainAll(stateLabels.SubList(0, number - 1));
				}
			}
			this.createTable();
			// Children must redo tables
			System.Collections.IEnumerator ite = children.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				((Node) ite.Current).createTable();
			}
		}
		
		public override void  toNet(System.IO.StreamWriter stream)
		{
			stream.Write("node " + this.getName() + "\n");
			stream.Write("{\n");
			if ((System.Object) this.label != null)
				stream.Write("    label = \"" + this.getLabel() + "\";\n");
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			stream.Write("    position = (" + (int) this.getPosition().getX() + " " + (int) this.getPosition().getY() + ");\n");
			stream.Write("    states = (");
			for (int i = 0; i < this.getNumberOfStates(); i++)
				stream.Write("\"" + this.getStateLabel(i) + "\" ");
			stream.Write(");\n");
			stream.Write("}\n\n");
		}
		
		public override void  copyNode(Node otherNode)
		{
			base.copyNode(otherNode);
			if (otherNode is DiscreteNode)
			{
				DiscreteNode node = (DiscreteNode) otherNode;
				try
				{
					this.setNumberOfStates(node.getNumberOfStates());
				}
				catch (MLPOperationNotAllowedException ex)
				{
					// An exception should never be thrown, but at least this makes sense
					// if it happens, nevertheless
					//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
					stateLabels = new System.Collections.ArrayList();
					//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					stateLabels.Add("State 0");
				}
				for (int i = 0; i < this.getNumberOfStates(); i++)
				{
					this.setStateLabel(i, node.getStateLabel(i));
				}
			}
		}
		
		public override void  toStream(System.String blanks, System.IO.StreamWriter stream)
		{
			base.toStream(blanks, stream);
			blanks = System.String.Concat(blanks, "  ");
			for (int i = 0, stop = this.getNumberOfStates(); i < stop; i++)
			{
				stream.Write(blanks + "<STATENAME>" + this.getStateLabel(i) + "</STATENAME>\n");
			}
			if (this is StateValues)
			{
				StateValues numbered = (StateValues) this;
				for (int i = 0, stop = this.getNumberOfStates(); i < stop; i++)
				{
					stream.Write(blanks + "<PROPERTY NAME =\"statevalue\">\n");
					blanks = System.String.Concat(blanks, "  ");
					stream.Write(blanks + "<PROPVALUE>" + numbered.getStateValue(i) + "</PROPVALUE>\n");
					blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
					stream.Write(blanks + "</PROPERTY>\n");
				}
			}
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</VAR>\n");
		}
	}
}