using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A clique of a compiled network.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class LimidClique : Drawable
	{
		/// <summary> Returns a list of the nodes being members of this clique.</summary>
		/// <returns>
		/// </returns>
		virtual public NodeList Members
		{
			get
			{
				return members;
			}
			
		}
		/// <summary> Used by the compilation routine.</summary>
		/// <returns> The index of this clique in the junction tree.
		/// </returns>
		virtual internal int NewIndex
		{
			get
			{
				return junctionTreeIndex;
			}
			
		}
		/// <summary> Returns the Limid potential of this clique.</summary>
		/// <returns> The Limid potential.
		/// </returns>
		virtual public LimidPotential Potential
		{
			get
			{
				return this.potential;
			}
			
		}
		/// <summary> Returns the list of decision potentials attached to this clique.</summary>
		/// <returns> A list of decision node potentials.
		/// </returns>
		virtual public SupportClass.ListCollectionSupport DecisionPotentials
		{
			get
			{
				return this.decisionPotentials;
			}
			
		}
		/// <summary> Returns a list of cliques being neighbors to this clique.</summary>
		/// <returns> A list of neighbor cliques.
		/// </returns>
		virtual public SupportClass.ListCollectionSupport Neighbors
		{
			get
			{
				return neighbors;
			}
			
		}
		/// <summary> Returns the list of separators.</summary>
		/// <returns> The list of separators.
		/// </returns>
		/// <seealso cref="LimidClique#separators">
		/// </seealso>
		virtual public SupportClass.ListCollectionSupport Separators
		{
			get
			{
				return separators;
			}
			
		}
		/// <summary> Returns the position of this clique.</summary>
		/// <returns> The position.
		/// </returns>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		/// <summary> Sets the position of this clique.</summary>
		/// <param name="pos">The position.
		/// </param>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		virtual public Point2D Position
		{
			get
			{
				return position;
			}
			
			set
			{
				position = value;
			}
			
		}
		/// <summary> The Limid potential of this clique</summary>
		protected internal LimidPotential potential;
		
		/// <summary> The member nodes of this clique.</summary>
		protected internal NodeList members;
		
		/// <summary> The nodes of the separator owned by this clique.</summary>
		private NodeList separatorNodes;
		
		/// <summary> The list of separators. There is one separator for each neighbor. The
		/// ordering of the separators must be in accordance with the ordering of the
		/// neighbors.
		/// </summary>
		protected internal SupportClass.ListCollectionSupport separators;
		
		/// <summary> The list of neighbors. The ordering of the neighbors must be in accordance
		/// with the ordering of the separators.
		/// </summary>
		protected internal SupportClass.ListCollectionSupport neighbors;
		
		/// <summary> A list of decision potentials. Before a propagation is performed, these
		/// potentials must be combined with the Limid potential.
		/// </summary>
		protected internal SupportClass.ListCollectionSupport decisionPotentials;
		
		/// <summary> The position of this clique. Used for drawing the junction tree.</summary>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		protected internal Point2D position;
		
		/// <summary> Used by the compilation routine.</summary>
		internal int junctionTreeIndex;
		
		/// <summary> Creates a clique. This is usually done by the API as part of the compilation
		/// procedure. It is not recommended to call the constructor diectly.
		/// </summary>
		/// <param name="m">The list of nodes that are members of this clique.
		/// </param>
		/// <param name="eliminated">A list of nodes already eliminated from the network.
		/// </param>
		/// <param name="present">A list of nodes not yet eliminated from the network.
		/// </param>
		/// <param name="startIndex">Used by the compilation procedure
		/// @throws MLPOperationNotAllowedException
		/// </param>
		protected internal LimidClique(NodeList m, NodeList eliminated, NodeList present, int startIndex)
		{
			members = m;
			neighbors = new SupportClass.ListCollectionSupport();
			separators = new SupportClass.ListCollectionSupport();
			NodeList extended = new NodeList(members);
			extended.AddAll(eliminated);
			separatorNodes = new NodeList(members);
			System.Collections.IEnumerator ite = members.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				DiscreteNode node = (DiscreteNode) ite.Current;
				// Can this node be eliminated?
				if (extended.ContainsAll(node.undirectedEdges))
				{
					separatorNodes.RemoveElement(node);
					eliminated.Add(node);
					present.RemoveElement(node);
				}
			}
			junctionTreeIndex = startIndex + members.Count - separatorNodes.Count;
			// Create potential tables
			this.potential = new LimidPotential(this.Members);
			decisionPotentials = new SupportClass.ListCollectionSupport();
			this.initialize();
		}
		
		// This method is only used by the constructor
		private void  initialize()
		{
			potential.initialize();
		}
		
		/// <summary> Creates and returns the separator owned by this clique.</summary>
		/// <returns> The separator owned by this clique.
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		public virtual LimidSeparator createSeparator()
		{
			if (separatorNodes.Count == 0)
				return null;
			return new LimidSeparator(this, separatorNodes);
		}
		
		/// <summary> Returns a string representation of this clique.</summary>
		/// <returns> A string representation of this clique.
		/// </returns>
		public override System.String ToString()
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.AbstractCollection.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			return "Clique[" + members + "]";
		}
		
		/// <summary> Adds a neighbor and the corresponding separator to this clique.</summary>
		/// <param name="neighbor">The neighbor clique.
		/// </param>
		/// <param name="sep">The separator corresponding to the neighbor.
		/// </param>
		public virtual void  addNeighborAndSeparator(LimidClique neighbor, LimidSeparator sep)
		{
			neighbors.Add(neighbor);
			separators.Add(sep);
		}
		
		/// <summary> Collects to this clique through recursive call of this method. A full
		/// collect propagation is performed.
		/// </summary>
		/// <param name="originator">Should be null if this clique is considered as the
		/// current root of the junction tree.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  collectToClique(LimidClique originator)
		{
			// Find the neighbors and separators
			SupportClass.ListCollectionSupport cl = this.Neighbors;
			SupportClass.ListCollectionSupport sep = this.Separators;
			// Iterate over neighbors
			System.Collections.IEnumerator ite = cl.GetEnumerator();
			System.Collections.IEnumerator jte = sep.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				// Find next neighbor
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique neighbor = (LimidClique) ite.Current;
				// Find the corresponding separator
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidSeparator separator = (LimidSeparator) jte.Current;
				// Ask neighbors to collect evidence unless the neighbor is the originator
				// of the request
				if (neighbor != originator)
				{
					neighbor.collectToClique(this);
					// Now the neighbor is updated: collect from it!
					// Ask separator to collect message
					separator.getMessage(this);
					// Process the mesage
					this.potential.combineWith(separator.UpdatePotential);
					
					// Debugging only
					//System.out.println("Collected to "+this);
					//System.out.print(""+this.getPotential());
				}
				// Continue to next neighbor (if any)
			}
		}
		
		/// <summary> Performs a partial collect propagation along a specified path to an other
		/// clique.
		/// </summary>
		/// <param name="path">A list of separators and cliques specifying the path.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  collectToCliqueAlongPath(SupportClass.ListCollectionSupport path)
		{
			
			// Iterate along path. The path holds elements alternating between
			// separators and cliques. The relevant separator always comes before the
			// corresponding clique.
			
			// There is nothing to do if the path is empty
			if (path.Count > 0)
			{
				System.Collections.IEnumerator ite = path.GetEnumerator();
				// Draw the two first elements - we assume that the path is not empty
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidSeparator oldSeparator = (LimidSeparator) ite.Current;
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique oldClique = (LimidClique) ite.Current;
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					LimidSeparator newSeparator = (LimidSeparator) ite.Current;
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					LimidClique newClique = (LimidClique) ite.Current;
					// Ask the separator to get message
					oldSeparator.getMessage(newClique);
					// Let the clique process the message
					newClique.Potential.combineWith(oldSeparator.UpdatePotential);
					// Redefine the "old" references
					oldSeparator = newSeparator;
					oldClique = newClique;
				}
				// Now we only need to handle this clique (which is implicitly understood
				// to be the last on the path).
				
				// Ask the separator to get the message to be received by this clique
				oldSeparator.getMessage(this);
				// Let this clique process the message
				this.Potential.combineWith(oldSeparator.UpdatePotential);
			}
		}
		
		/// <summary> Identifies the unique path in the junction tree from this clique
		/// to an other specified clique. The path is a list of separators alternating
		/// with cliques. The method calls itself recursively.
		/// </summary>
		/// <param name="originator">Used in the recursion. Should be null.
		/// </param>
		/// <param name="otherClique">The target clique.
		/// </param>
		/// <returns> A list of separators and cliques.
		/// </returns>
		public virtual SupportClass.ListCollectionSupport getPathToClique(LimidClique originator, LimidClique otherClique)
		{
			SupportClass.ListCollectionSupport path = new SupportClass.ListCollectionSupport();
			// It may happen that both decisions are in the same clique. In that case
			// an empty path is returned
			if (originator == otherClique)
				return path;
			// Otherwise, find the non-empty path
			SupportClass.ListCollectionSupport cl = this.Neighbors;
			SupportClass.ListCollectionSupport sl = this.Separators;
			// Iterate over neighbors - width first search
			bool notFound = true;
			System.Collections.IEnumerator ite = cl.GetEnumerator();
			System.Collections.IEnumerator jte = sl.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext() && notFound)
			{
				// Find next neighbor
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique neighbor = (LimidClique) ite.Current;
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidSeparator separator = (LimidSeparator) jte.Current;
				// Is neighbor the desired clique?
				if (neighbor == otherClique)
				{
					path.Add(separator);
					path.Add(neighbor);
					notFound = false;
				}
			}
			// The other clique is not a direct neighbor - extend depth
			if (notFound)
			{
				ite = cl.GetEnumerator();
				jte = sl.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext() && notFound)
				{
					// Find next neighbor
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					LimidClique neighbor = (LimidClique) ite.Current;
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					LimidSeparator separator = (LimidSeparator) jte.Current;
					// Don't go back
					if (neighbor != originator)
					{
						path = neighbor.getPathToClique(this, otherClique);
						if (path.Count > 0)
						{
							// Add separator and clique to path
							path.Add(separator);
							path.Add(neighbor);
							notFound = false;
						}
					}
				}
			}
			return path;
		}
		
		/// <summary> Adds a decision node potential to this clique.</summary>
		/// <param name="decision">The decisionn node potential
		/// </param>
		/// <seealso cref="LimidClique#decisionPotentials">
		/// </seealso>
		public virtual void  addDecisionPotential(Potential decision)
		{
			decisionPotentials.Add(decision);
		}
		
		/// <summary> Returns the geometric size of this clique when drawn to a graphics context.</summary>
		/// <param name="graphics">The graphics context.
		/// </param>
		/// <returns> The size.
		/// </returns>
		public virtual System.Drawing.Size getSize(System.Drawing.Graphics graphics)
		{
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.FontMetrics.getStringBounds' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			int lineHeight = (int) SupportClass.GraphicsManager.manager.GetFont(graphics).getStringBounds("Wgj", graphics).getHeight() + 5;
			System.Collections.IEnumerator ite = this.members.GetEnumerator();
			int count = 0;
			double width = 0.0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.awt.FontMetrics.getStringBounds' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				double w = SupportClass.GraphicsManager.manager.GetFont(graphics).getStringBounds(node.ToString(), graphics).getWidth();
				if (w > width)
					width = w;
				count++;
			}
			int boxHeigth = lineHeight * members.Count + 10;
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			int boxWidth = (int) width + 10;
			return new System.Drawing.Size(boxWidth, boxHeigth);
		}
		
		/// <summary> Determines the position in the junction tree grid of this clique given the
		/// parent and the parent's position in the grid. Used internally when drawing
		/// a junction tree. Should not be called directly.
		/// </summary>
		/// <param name="parent">The parent clique.
		/// </param>
		/// <param name="parentPos">The parent's position in the grid.
		/// </param>
		/// <param name="grid">The grid used for positioning.
		/// </param>
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		protected internal virtual void  posision(LimidClique parent, System.Drawing.Point parentPos, System.Collections.ArrayList grid)
		{
			System.Drawing.Point pos = new System.Drawing.Point(parentPos.X + 1, 0);
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			if (pos.X >= grid.Count)
			{
				// Create new column
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList column = new System.Collections.ArrayList();
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				column.Add(this);
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				grid.Add(column);
				pos = new System.Drawing.Point(pos.X, 0);
			}
			else
			{
				// Add to existing column
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList column = (System.Collections.ArrayList) grid[pos.X];
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				column.Add(this);
				pos = new System.Drawing.Point(pos.X, column.IndexOf(this));
			}
			// Now, call recursively on all other neighbors than the "parent"
			System.Collections.IEnumerator ite = this.neighbors.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique neighbor = (LimidClique) ite.Current;
				if (neighbor != parent)
					neighbor.posision(this, pos, grid);
			}
		}
		
		public virtual void  draw(System.Drawing.Graphics graphics)
		{
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.FontMetrics.getStringBounds' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			int lineHeight = (int) SupportClass.GraphicsManager.manager.GetFont(graphics).getStringBounds("Wgj", graphics).getHeight() + 5;
			System.Drawing.Size dimension = this.getSize(graphics);
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.LightGray);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Graphics.fillRoundRect' was converted to 'System.Drawing.Graphics.FillRectangle' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javaawtGraphicsfillRoundRect_int_int_int_int_int_int"';;
			graphics.FillRectangle(SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) position.getX(), (int) position.getY(), dimension.Width, dimension.Height);;;
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
			//UPGRADE_TODO: Method 'java.awt.Graphics.drawRoundRect' was converted to 'System.Drawing.Graphics.DrawRectangle' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javaawtGraphicsdrawRoundRect_int_int_int_int_int_int"'
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.DrawRectangle(SupportClass.GraphicsManager.manager.GetPen(graphics), (int) position.getX(), (int) position.getY(), dimension.Width, dimension.Height);
			int count = 0;
			System.Collections.IEnumerator ite = members.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				//UPGRADE_TODO: Method 'java.awt.Graphics.drawString' was converted to 'System.Drawing.Graphics.DrawString' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javaawtGraphicsdrawString_javalangString_int_int"'
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				graphics.DrawString(node.ToString(), SupportClass.GraphicsManager.manager.GetFont(graphics), SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) position.getX() + 5, (int) position.getY() + lineHeight * (count + 1) - SupportClass.GraphicsManager.manager.GetFont(graphics).GetHeight());
				count++;
			}
			// Find "parent" and add link;
			ite = neighbors.GetEnumerator();
			bool notFound = true;
			LimidClique parent = null;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique clique = (LimidClique) ite.Current;
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				if (clique.Position != null && clique.Position.getX() < this.Position.getX() - 5.0)
				{
					notFound = false;
					parent = clique;
				}
			}
			if (parent != null)
			{
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				int xAnchor = (int) parent.Position.getX() + parent.getSize(graphics).Width;
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				int yAnchor = (int) parent.Position.getY() + parent.getSize(graphics).Height / 2;
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				int xTarget = (int) position.getX();
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
				int yTarget = (int) position.getY() + dimension.Height / 2;
				graphics.DrawLine(SupportClass.GraphicsManager.manager.GetPen(graphics), xAnchor, yAnchor, xTarget, yTarget);
			}
		}
		
		public virtual void  drawSelected(System.Drawing.Graphics graphics)
		{
			this.draw(graphics);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual bool isPointWithinShape(Point2D point)
		{
			return false;
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Drawable getSelectedItem(Point2D point)
		{
			return null;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready)
		{
			return allready;
		}
	}
}