using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The class representing the junction tree of a compiled Limid.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class LimidJunctionTree : Drawable
	{
		/// <summary> Returns the list of cliques.</summary>
		/// <returns> The list of cliques.
		/// </returns>
		virtual public SupportClass.ListCollectionSupport Cliques
		{
			get
			{
				return this.cliques;
			}
			
		}
		
		/// <summary> The list of cliques of the junction tree.</summary>
		protected internal SupportClass.ListCollectionSupport cliques;
		
		/// <summary> The list of separators of the junction tree</summary>
		protected internal SupportClass.ListCollectionSupport separators;
		
		/// <summary> The Limid owning this junction tree.</summary>
		protected internal LimidDomain model;
		
		/// <summary> Creates the junction tree from the Limid</summary>
		/// <param name="model">The Limid
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public LimidJunctionTree(LimidDomain model)
		{
			this.model = model;
			cliques = new SupportClass.ListCollectionSupport();
			separators = new SupportClass.ListCollectionSupport();
			NodeList modelNodes = new NodeList(model.Nodes);
			// Ignore utility nodes
			SupportClass.ListCollectionSupport utilities = new SupportClass.ListCollectionSupport();
			System.Collections.IEnumerator iterator = modelNodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (iterator.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) iterator.Current;
				if (node is TimeUtilityNode)
					utilities.Add(node);
			}
			modelNodes.RemoveAll(utilities);
			NodeList eliminated = new NodeList();
			int allNodes = modelNodes.Count;
			int index = 0;
			
			while (eliminated.Count < allNodes)
			{
				// Find simplicial node
				System.Collections.IEnumerator ite = modelNodes.GetEnumerator();
				DiscreteNode simplicial = (DiscreteNode) modelNodes[0];
				bool notFound = true;
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext() && notFound)
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode node = (DiscreteNode) ite.Current;
					if (node.isSimplicialWithin(modelNodes))
					{
						notFound = false;
						simplicial = node;
					}
				}
				NodeList family = new NodeList();
				ite = simplicial.undirectedEdges.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					DiscreteNode node = (DiscreteNode) ite.Current;
					if (!eliminated.Contains(node))
						family.Add(node);
				}
				family.Add(simplicial);
				LimidClique clique = new LimidClique(family, eliminated, modelNodes, index);
				int newIndex = clique.NewIndex;
				cliques.Add(clique);
				LimidSeparator separator = clique.createSeparator();
				if (separator != null)
					separators.Add(separator);
				index = newIndex;
			}
			
			// Now the sets of cliques and separators have been constructed
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			System.Collections.ArrayList cli = SupportClass.CreateCollection(this.cliques);
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			System.Collections.ArrayList sep = SupportClass.CreateCollection(this.separators);
			int i = 0;
			int j = 0;
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			while (i < sep.Count)
			{
				LimidSeparator sepe = (LimidSeparator) sep[i];
				bool notFound = true;
				j = i + 1;
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				while (j < cli.Count && notFound)
				{
					LimidClique cliq = (LimidClique) cli[j];
					if (cliq.Members.ContainsAll(sepe.SeparatorMembers))
					{
						notFound = false;
						LimidClique left = (LimidClique) cli[i];
						// Is the separator identical to the target clique
						if (sepe.SeparatorMembers.ContainsAll(cliq.Members))
						{
							// In that case delete the target
							System.Object item;
							item = cli[j];
							cli[j] = left;
							System.Object generatedAux = item;
							cliques.RemoveElement(cliq);
							separators.RemoveElement(sepe);
						}
						else
						{
							sepe.setRightClique(cliq);
							// The next line is necessary because the present left clique may
							// have been a subset of an other clique that have absorbed it
							sepe.setLeftClique(left);
							left.addNeighborAndSeparator(cliq, sepe);
							cliq.addNeighborAndSeparator(left, sepe);
						}
					}
					j++;
				}
				i++;
			}
			this.initialize(model.Nodes);
		}
		
		/// <summary> Initialize the junction tree. Called by the constructor. The initial
		/// tables of all cliques are calculated, and the decision node potentials
		/// are added to the relevant cliques.
		/// </summary>
		/// <param name="nodes">The nodes of the Limid.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  initialize(NodeList nodes)
		{
			System.Collections.IEnumerator ite = nodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node thisNode = (Node) ite.Current;
				// First process discrete chance nodes
				if (thisNode is DiscreteNode)
				{
					DiscreteNode node = (DiscreteNode) thisNode;
					PotentialTable nodeTable = new PotentialTable(node.Table);
					NodeList potentialNodes = node.Table.Nodes;
					// Find a clique containing all potential nodes
					System.Collections.IEnumerator j = this.Cliques.GetEnumerator();
					bool notFound = true;
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (j.MoveNext() && notFound)
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						LimidClique lc = (LimidClique) j.Current;
						// Does the clique contain all nodes of the potential?
						if (lc.Members.ContainsAll(potentialNodes))
						{
							// Yes, we have found such a clique, stop the iteration
							notFound = false;
							
							// Debugging only
							// System.out.println("Node "+thisNode+" combined with "+lc);
							
							// Multiply the potentials if not decision
							if (node is DiscreteDecisionChanceNode)
							{
								DiscreteDecisionChanceNode ddcn = (DiscreteDecisionChanceNode) node;
								lc.addDecisionPotential(ddcn.Table);
								ddcn.HomeClique = lc;
							}
							else
								lc.Potential.combineWithProbabilities(nodeTable);
							
							// Debugging only
							// System.out.print(""+lc.getPotential());
						}
					}
				}
				// Then, process utility nodes
				else
				{
					TimeUtilityNode node = (TimeUtilityNode) thisNode;
					PotentialTable nodeTable = new PotentialTable(node.Table);
					NodeList potentialNodes = node.Table.Nodes;
					// Find a clique containing all potential nodes
					System.Collections.IEnumerator j = this.Cliques.GetEnumerator();
					bool notFound = true;
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (j.MoveNext() && notFound)
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						LimidClique lc = (LimidClique) j.Current;
						// Does the clique contain all nodes of the potential?
						if (lc.Members.ContainsAll(potentialNodes))
						{
							// Yes, we have found such a clique, stop the iteration
							notFound = false;
							
							// Debugging only
							//System.out.println("Node "+thisNode+" combined with "+lc);
							
							// Multiply the potentials
							lc.Potential.combineWithUtilities(nodeTable);
							
							// Debugging only
							// System.out.print(""+lc.getPotential());
						}
					}
				}
			}
		}
		
		
		
		/// <summary> returns a string representation of the junction tree.</summary>
		/// <returns> A string representation of the junction tree.
		/// </returns>
		public override System.String ToString()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder("JunctionTree[\n");
			System.Collections.IEnumerator ite = cliques.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique lc = (LimidClique) ite.Current;
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				sb.Append(lc + ", Neighbors : ");
				SupportClass.ListCollectionSupport neighbors = lc.Neighbors;
				System.Collections.IEnumerator jte = neighbors.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (jte.MoveNext())
				{
					//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					sb.Append(jte.Current + ", ");
				}
				sb.Append("\n");
			}
			sb.Append("]\n");
			return sb.ToString();
		}
		
		public virtual void  draw(System.Drawing.Graphics graphics)
		{
			// Find leaf clique
			System.Collections.IEnumerator ite = this.cliques.GetEnumerator();
			LimidClique leafClique = null;
			bool notFound = true;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext() && notFound)
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				LimidClique clique = (LimidClique) ite.Current;
				if (clique.Neighbors.Count < 2)
				{
					leafClique = clique;
					notFound = false;
				}
			}
			// Establish a clique grid as a list of columns
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			System.Collections.ArrayList grid = new System.Collections.ArrayList();
			leafClique.posision(null, new System.Drawing.Point(- 1, 0), grid);
			// Set clique positions and draw
			int xPos = 10;
			int nextX = 10;
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0; i < grid.Count; i++)
			{
				xPos = nextX;
				//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
				System.Collections.ArrayList column = (System.Collections.ArrayList) grid[i];
				int yPos = 10;
				int nextY = 10;
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				for (int j = 0; j < column.Count; j++)
				{
					yPos = nextY;
					LimidClique clique = (LimidClique) column[j];
					System.Drawing.Size dim = clique.getSize(graphics);
					if (xPos + dim.Width + 40 > nextX)
						nextX = xPos + dim.Width + 40;
					nextY = yPos + dim.Height + 40;
					clique.Position = new System.Drawing.Point(xPos, yPos);
					clique.draw(graphics);
				}
			}
		}
		
		public virtual void  drawSelected(System.Drawing.Graphics graphics)
		{
			draw(graphics);
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual bool isPointWithinShape(Point2D point)
		{
			return false;
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Drawable getSelectedItem(Point2D point)
		{
			return null;
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready)
		{
			return allready;
		}
	}
}