using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A class representing the Limid potential of a clique or a separator. Basically
	/// a Limid potential holds a probability table and a utility table. The methods
	/// of the class support the algebra of Limid potentials.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class LimidPotential
	{
		/// <summary> Returns the probability table.</summary>
		/// <returns> The probability table.
		/// </returns>
		virtual public Potential ProbabilityTable
		{
			get
			{
				return this.probabilityTable;
			}
			
		}
		/// <summary> Returns the utility table.</summary>
		/// <returns> The utility table.
		/// </returns>
		virtual public Potential UtilityTable
		{
			get
			{
				return this.utilityTable;
			}
			
		}
		/// <summary> Returns the contraction (element-by-element product) of the two tables.</summary>
		/// <returns> The contraction as a table.
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		virtual public PotentialTable Contraction
		{
			get
			{
				double[] probs = this.ProbabilityTable.Data;
				double[] uts = this.UtilityTable.Data;
				double[] ct = new double[probs.Length];
				for (int i = 0; i < ct.Length; i++)
					ct[i] = probs[i] * uts[i];
				PotentialTable table = new PotentialTable(probabilityTable.Nodes);
				table.Data = ct;
				return table;
			}
			
		}
		/// <summary> This method calculates the expected utility based on the probability table
		/// and the utility table. If it is called on the Limid potential of the current
		/// root of the junction tree after a collect propagation, the value returned
		/// is the expected utility of the entire model.
		/// </summary>
		/// <returns> The expected utility.
		/// </returns>
		virtual public double ExpectedUtility
		{
			get
			{
				double[] pro = this.probabilityTable.Data;
				double[] uts = this.utilityTable.Data;
				double sum1 = 0.0;
				double sum2 = 0.0;
				for (int i = 0; i < pro.Length; i++)
				{
					sum1 = sum1 + pro[i] * uts[i];
					sum2 = sum2 + pro[i];
				}
				return (sum1 / sum2);
			}
			
		}
		
		/// <summary> The probability table</summary>
		protected internal Potential probabilityTable;
		
		/// <summary> The utility table</summary>
		protected internal Potential utilityTable;
		
		/// <summary> Creates an (empty) Limid potential from a list of nodes.</summary>
		/// <param name="nodes">The nodes defining the configuration of the clique or separator.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public LimidPotential(NodeList nodes)
		{
			probabilityTable = new PotentialTable(nodes);
			utilityTable = new PotentialTable(nodes);
		}
		
		/// <summary> Creates a Limid potential from a probability table and a utility table.</summary>
		/// <param name="pTable">The probability table
		/// </param>
		/// <param name="uTable">The utility table.
		/// @throws MLPNotSameDomainException
		/// </param>
		public LimidPotential(Potential pTable, Potential uTable)
		{
			if (!SupportClass.EqualsSupport(pTable.Nodes, uTable.Nodes))
				throw new MLPNotSameDomainException();
			probabilityTable = pTable;
			utilityTable = uTable;
		}
		
		/// <summary> Initializes both tables by putting all probabilities equal to 1 and all
		/// utilities equal to 0.
		/// </summary>
		public virtual void  initialize()
		{
			double[] pValues = new double[probabilityTable.Size];
			double[] uValues = new double[probabilityTable.Size];
			for (int i = 0; i < pValues.Length; i++)
			{
				pValues[i] = 1.0;
				uValues[i] = 0.0;
			}
			probabilityTable.Data = pValues;
			utilityTable.Data = uValues;
		}
		
		/// <summary> Marginalize the Limid potential to be expressed only by the specified nodes.</summary>
		/// <param name="keepNodes">The nodes being "kept" in the configuration.
		/// </param>
		/// <returns> The marginalized Limid potential
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		public virtual LimidPotential marginalize(NodeList keepNodes)
		{
			// Find indexes of these nodes in the this table's node list. There are
			// only few nodes in a clique so the algoritmic inefficiency of the
			// following procedure does not matter.
			int[] indexes = new int[keepNodes.Count];
			int j = 0;
			System.Collections.IEnumerator ite = keepNodes.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				indexes[j] = probabilityTable.Nodes.IndexOf((Node) ite.Current);
				j++;
			}
			// Create output tables
			PotentialTable newProbabilityTable = new PotentialTable(keepNodes);
			PotentialTable newUtilityTable = new PotentialTable(keepNodes);
			LimidPotential marginalPotential = new LimidPotential(newProbabilityTable, newUtilityTable);
			double[] pValues = new double[newProbabilityTable.Size];
			double[] uValues = new double[newProbabilityTable.Size];
			int[] state = new int[0];
			int[] keepState = new int[indexes.Length];
			int nc = indexes.Length;
			int index = 0;
			this.probabilityTable.initialize();
			while (this.probabilityTable.hasNextState())
			{
				state = this.probabilityTable.nextState();
				for (int i = 0; i < nc; i++)
					keepState[i] = state[indexes[i]];
				index = newProbabilityTable.convertToIndex(keepState);
				int tableIndex = this.probabilityTable.convertToIndex(state);
				pValues[index] = pValues[index] + this.probabilityTable.getDataItem(tableIndex);
				uValues[index] = uValues[index] + this.utilityTable.getDataItem(tableIndex) * this.probabilityTable.getDataItem(tableIndex);
			}
			// Normalize ...
			for (int i = 0; i < uValues.Length; i++)
			{
				if (pValues[i] == 0d)
					uValues[i] = 0.0;
				else
					uValues[i] = uValues[i] / pValues[i];
			}
			newProbabilityTable.Data = pValues;
			newUtilityTable.Data = uValues;
			return marginalPotential;
		}
		
		/// <summary> Combines the probability table with an other probability table. The original
		/// probability table is kept unchanged. Basically, the probabilities are
		/// multiplied.
		/// </summary>
		/// <param name="nodeTable">The other probability table.
		/// </param>
		/// <returns> A Limid potential with updated probability table.
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		public virtual LimidPotential combineCopyWithProbabilities(Potential nodeTable)
		{
			// Multiply the potentials
			// Clone the probability table
			PotentialTable probs = new PotentialTable(probabilityTable.Nodes);
			int[] cliqueState = new int[0];
			NodeList potentialNodes = nodeTable.Nodes;
			// Find the mapping of clique state to node state.  There are
			// only few nodes in a clique so the algoritmic inefficiency of the
			// following procedure does not matter.
			System.Collections.IEnumerator k = potentialNodes.GetEnumerator();
			int[] nodeState = new int[potentialNodes.Count];
			int[] cliqueIndex = new int[potentialNodes.Count];
			int j = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (k.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node tableNode = (Node) k.Current;
				cliqueIndex[j] = this.ProbabilityTable.Nodes.IndexOf(tableNode);
				j++;
			}
			// Iterate over all clique states
			this.ProbabilityTable.initialize();
			while (this.ProbabilityTable.hasNextState())
			{
				cliqueState = this.ProbabilityTable.nextState();
				// Find the relevant node state
				for (int l = 0; l < nodeState.Length; l++)
					nodeState[l] = cliqueState[cliqueIndex[l]];
				// Now, fetch the probability and update value
				int index = this.ProbabilityTable.convertToIndex(cliqueState);
				double value_Renamed = nodeTable.getDataItem(nodeState) * this.ProbabilityTable.getDataItem(index);
				probs.setDataItem(index, value_Renamed);
			}
			return (new LimidPotential(probs, utilityTable));
		}
		
		/// <summary> Combines the probability table with an other probability table. The original
		/// probability table is replaced by the combined result. Basically, the
		/// probabilities are multiplied.
		/// </summary>
		/// <param name="nodeTable">The other probability table.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  combineWithProbabilities(PotentialTable nodeTable)
		{
			// Multiply the potentials
			// Clone the probability table
			int[] cliqueState = new int[0];
			NodeList potentialNodes = nodeTable.Nodes;
			// Find the mapping of clique state to node state. There are
			// only few nodes in a clique so the algoritmic inefficiency of the
			// following procedure does not matter.
			System.Collections.IEnumerator k = potentialNodes.GetEnumerator();
			int[] nodeState = new int[potentialNodes.Count];
			int[] cliqueIndex = new int[potentialNodes.Count];
			int j = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (k.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node tableNode = (Node) k.Current;
				cliqueIndex[j] = this.ProbabilityTable.Nodes.IndexOf(tableNode);
				j++;
			}
			// Iterate over all clique states
			this.ProbabilityTable.initialize();
			while (this.ProbabilityTable.hasNextState())
			{
				cliqueState = this.ProbabilityTable.nextState();
				// Find the relevant node state
				for (int l = 0; l < nodeState.Length; l++)
					nodeState[l] = cliqueState[cliqueIndex[l]];
				// Now, fetch the probability and update value
				int index = this.ProbabilityTable.convertToIndex(cliqueState);
				double value_Renamed = nodeTable.getDataItem(nodeState) * this.ProbabilityTable.getDataItem(index);
				this.ProbabilityTable.setDataItem(index, value_Renamed);
			}
		}
		
		/// <summary> Combines the utility table with an other utility table. The original
		/// utility table is replaced by the combined result. Basically, the
		/// utilities are summed.
		/// </summary>
		/// <param name="nodeTable">The other utility table.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  combineWithUtilities(PotentialTable nodeTable)
		{
			// Multiply the potentials
			int[] cliqueState = new int[0];
			NodeList potentialNodes = nodeTable.Nodes;
			// Find the mapping of clique state to node state.  There are
			// only few nodes in a clique so the algoritmic inefficiency of the
			// following procedure does not matter.
			System.Collections.IEnumerator k = potentialNodes.GetEnumerator();
			int[] nodeState = new int[potentialNodes.Count];
			int[] cliqueIndex = new int[potentialNodes.Count];
			int j = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (k.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node tableNode = (Node) k.Current;
				cliqueIndex[j] = this.UtilityTable.Nodes.IndexOf(tableNode);
				j++;
			}
			// Iterate over all clique states
			this.UtilityTable.initialize();
			while (this.UtilityTable.hasNextState())
			{
				cliqueState = this.UtilityTable.nextState();
				// Find the relevant node state
				for (int l = 0; l < nodeState.Length; l++)
					nodeState[l] = cliqueState[cliqueIndex[l]];
				// Now, fetch the probability and update value
				int index = this.UtilityTable.convertToIndex(cliqueState);
				double value_Renamed = nodeTable.getDataItem(nodeState) + this.UtilityTable.getDataItem(index);
				this.UtilityTable.setDataItem(index, value_Renamed);
			}
		}
		
		/// <summary> Combines this Limid potential with an other. (Probabilities are multiplied,
		/// utilities are summed. The original tables are replaced by the results.
		/// </summary>
		/// <param name="otherPotential">The other Limid potential.
		/// </param>
		public virtual void  combineWith(LimidPotential otherPotential)
		{
			// Multiply the potentials
			int[] cliqueState = new int[0];
			NodeList potentialNodes = otherPotential.ProbabilityTable.Nodes;
			// Find the mapping of clique state to node state.  There are
			// only few nodes in a clique so the algoritmic inefficiency of the
			// following procedure does not matter.
			System.Collections.IEnumerator k = potentialNodes.GetEnumerator();
			int[] nodeState = new int[potentialNodes.Count];
			int[] cliqueIndex = new int[potentialNodes.Count];
			int j = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (k.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node tableNode = (Node) k.Current;
				cliqueIndex[j] = this.ProbabilityTable.Nodes.IndexOf(tableNode);
				j++;
			}
			// Iterate over all clique states
			this.ProbabilityTable.initialize();
			while (this.ProbabilityTable.hasNextState())
			{
				cliqueState = this.ProbabilityTable.nextState();
				// Find the relevant node state
				for (int l = 0; l < nodeState.Length; l++)
					nodeState[l] = cliqueState[cliqueIndex[l]];
				// Now, fetch the probability and update value
				int index = this.ProbabilityTable.convertToIndex(cliqueState);
				double pValue = otherPotential.ProbabilityTable.getDataItem(nodeState) * this.ProbabilityTable.getDataItem(index);
				this.ProbabilityTable.setDataItem(index, pValue);
				double uValue = otherPotential.UtilityTable.getDataItem(nodeState) + this.UtilityTable.getDataItem(index);
				this.UtilityTable.setDataItem(index, uValue);
			}
		}
		
		/// <summary> Division of Limid potentials. The present Limid potential is divided by
		/// the other Limid potential. (Probabilities are divided - under the covention
		/// 0/0 = 0 - and utilities are subtracted). The original tables are replaced by
		/// the results.
		/// </summary>
		/// <param name="otherPotential">The other Limid potential.
		/// </param>
		public virtual void  divideBy(LimidPotential otherPotential)
		{
			// Divide the potentials
			int[] cliqueState = new int[0];
			NodeList potentialNodes = otherPotential.ProbabilityTable.Nodes;
			// Find the mapping of clique state to node state.  There are
			// only few nodes in a clique so the algoritmic inefficiency of the
			// following procedure does not matter.
			System.Collections.IEnumerator k = potentialNodes.GetEnumerator();
			int[] nodeState = new int[potentialNodes.Count];
			int[] cliqueIndex = new int[potentialNodes.Count];
			int j = 0;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (k.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node tableNode = (Node) k.Current;
				cliqueIndex[j] = this.ProbabilityTable.Nodes.IndexOf(tableNode);
				j++;
			}
			// Iterate over all clique states
			this.ProbabilityTable.initialize();
			while (this.ProbabilityTable.hasNextState())
			{
				cliqueState = this.ProbabilityTable.nextState();
				// Find the relevant node state
				for (int l = 0; l < nodeState.Length; l++)
					nodeState[l] = cliqueState[cliqueIndex[l]];
				// Now, fetch the probability and update value
				int index = this.ProbabilityTable.convertToIndex(cliqueState);
				double pValue;
				if (otherPotential.ProbabilityTable.getDataItem(nodeState) == 0d)
					pValue = 0.0;
				else
					pValue = this.ProbabilityTable.getDataItem(index) / otherPotential.ProbabilityTable.getDataItem(nodeState);
				this.ProbabilityTable.setDataItem(index, pValue);
				double uValue = this.UtilityTable.getDataItem(index) - otherPotential.UtilityTable.getDataItem(nodeState);
				this.UtilityTable.setDataItem(index, uValue);
			}
		}
		
		/// <summary> This method basically does the same as the divideBy method, but if the two
		/// Limid potentials are defined in the same domain, this method is much more
		/// efficient.
		/// </summary>
		/// <param name="otherPotential">The other Limid potential
		/// </param>
		/// <returns> The result of the operation.
		/// @throws MLPOperationNotAllowedException
		/// </returns>
		public virtual LimidPotential divideByInSameDomain(LimidPotential otherPotential)
		{
			if (!SupportClass.EqualsSupport(this.ProbabilityTable.Nodes, otherPotential.ProbabilityTable.Nodes))
				throw new MLPNotSameDomainException();
			double[] old = this.probabilityTable.Data;
			double[] oldUtValues = this.utilityTable.Data;
			double[] newValues = otherPotential.ProbabilityTable.Data;
			double[] newUtValues = otherPotential.UtilityTable.Data;
			double[] ratios = new double[old.Length];
			double[] utDifferences = new double[old.Length];
			// Now, fetch the probability and
			for (int i = 0; i < old.Length; i++)
			{
				if (old[i] == 0d)
					ratios[i] = 0;
				else
					ratios[i] = old[i] / newValues[i];
				utDifferences[i] = oldUtValues[i] - newUtValues[i];
			}
			PotentialTable rates = new PotentialTable(this.ProbabilityTable.Nodes);
			PotentialTable uts = new PotentialTable(this.UtilityTable.Nodes);
			rates.Data = ratios;
			uts.Data = utDifferences;
			return new LimidPotential(rates, uts);
		}
		
		public override System.String ToString()
		{
			return probabilityTable.toString(utilityTable);
		}
	}
}