using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The class representing a separator of the junction tree.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class LimidSeparator
	{
		/// <summary> Returns the update potential of this separator.</summary>
		/// <returns> The update potential of this separator.
		/// </returns>
		virtual public LimidPotential UpdatePotential
		{
			get
			{
				return updatePotential;
			}
			
		}
		/// <summary> Returns the nodes of this separator.</summary>
		/// <returns> A list of member nodes.
		/// </returns>
		virtual public NodeList SeparatorMembers
		{
			get
			{
				return common;
			}
			
		}
		
		/// <summary> The "left" clique. The left clique is considered as owner of the separator.</summary>
		protected internal LimidClique left;
		
		/// <summary> The "right" clique.</summary>
		protected internal LimidClique right;
		
		/// <summary> The node list of the separator.</summary>
		protected internal NodeList common;
		
		/// <summary> The Limid potential of the separator.</summary>
		protected internal LimidPotential separatorPotential;
		
		/// <summary> The update potential of the separator.</summary>
		protected internal LimidPotential updatePotential;
		
		
		/// <summary> Creates a separator from a specified owner and a list of nodes constituing
		/// the common nodes.
		/// </summary>
		/// <param name="left">The owner clique
		/// </param>
		/// <param name="common">The list of common nodes
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public LimidSeparator(LimidClique left, NodeList common)
		{
			this.left = left;
			this.common = common;
			separatorPotential = new LimidPotential(this.SeparatorMembers);
			updatePotential = new LimidPotential(this.SeparatorMembers);
			this.initialize();
		}
		
		/// <summary> Defines the right clique of this separator. The method is used by the API.
		/// It should not be called directly. No validation is performed.
		/// </summary>
		/// <param name="right">The right clique.
		/// </param>
		internal virtual void  setRightClique(LimidClique right)
		{
			this.right = right;
		}
		
		/// <summary> Defines the left clique of this separator. The method is used by the API.
		/// It should not be called directly. No validation is performed.
		/// </summary>
		/// <param name="left">The left clique.
		/// </param>
		internal virtual void  setLeftClique(LimidClique left)
		{
			this.left = left;
		}
		
		/// <summary> Returns the right clique of this separator.</summary>
		/// <returns> The right clique.
		/// </returns>
		public virtual LimidClique getRightClique()
		{
			return right;
		}
		
		/// <summary> Returns the left clique of this separator.</summary>
		/// <returns> The left clique.
		/// </returns>
		public virtual LimidClique getLeftClique()
		{
			return left;
		}
		
		/// <summary> Initializes the potential tables. Used by the constructor.</summary>
		internal virtual void  initialize()
		{
			separatorPotential.initialize();
			updatePotential.initialize();
		}
		
		/// <summary> Collects messages from the clique opposite to the caller.</summary>
		/// <param name="caller">The clique that demands a message.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  getMessage(LimidClique caller)
		{
			LimidClique destination = right;
			// In what direction do we receive
			if (caller == left)
			{
				destination = right;
			}
			else
			{
				destination = left;
			}
			
			// Multiply with decision potentials before retrieving message
			LimidPotential destinationPotential = destination.Potential;
			System.Collections.IEnumerator ite = destination.DecisionPotentials.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Potential nodeTable = (Potential) ite.Current;
				destinationPotential = destinationPotential.combineCopyWithProbabilities(nodeTable);
			}
			
			// Debugging only
			// System.out.print("Updated destination "
			//                 +destination+":\n"+destinationPotential);
			
			LimidPotential marginals = destinationPotential.marginalize(this.SeparatorMembers);
			updatePotential = marginals.divideByInSameDomain(separatorPotential);
			separatorPotential = marginals;
			
			// Debugging only
			// System.out.println("Separator potential "+
			//                    this+":\n"+separatorPotential);
			// System.out.println("Update potential "
			//                    +this+":\n"+updatePotential);
		}
		
		/// <summary> Returns a string representation of this separator.</summary>
		/// <returns> A string representation of this separator.
		/// </returns>
		public override System.String ToString()
		{
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.AbstractCollection.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			return "Separator[" + common + "]";
		}
	}
}