using System;
namespace dk.esthauge.limid.api
{
	
	
	/// <summary> The abstract super class of all nodes.
	/// 
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public abstract class Node : Drawable
	{
		/// <summary> Returns the domain that owns this node.</summary>
		/// <returns> The home domain.
		/// </returns>
		virtual public LimidDomain HomeDomain
		{
			get
			{
				return this.domain;
			}
			
		}
		/// <summary> Returns the parents of this node.</summary>
		/// <returns> A list of parent nodes.
		/// </returns>
		virtual public NodeList Parents
		{
			get
			{
				return parents;
			}
			
		}
		/// <summary> Returns a list containing all children of this node.</summary>
		/// <returns> A list of child nodes.
		/// </returns>
		virtual public NodeList Children
		{
			get
			{
				return this.children;
			}
			
		}
		/// <summary> Returns the set of directed edges of this node.</summary>
		/// <returns> The edge set.
		/// </returns>
		virtual public EdgeSet DirectedEdges
		{
			get
			{
				return directed;
			}
			
		}
		/// <summary> Moves this node to an other domain.</summary>
		/// <param name="otherLimid">The other domain
		/// @throws MLPDuplicateNodeNameException
		/// @throws MLPOperationNotAllowedException
		/// </param>
		virtual public LimidDomain Limid
		{
			set
			{
				// Add node to other Limid
				value.addNode(this);
				// Remove from original Limid's node list
				SupportClass.HashtableRemove(domain.nodes, this.getName());
				// Inform node about new owner
				domain = value;
			}
			
		}
		/// <summary> Returns the potential table of this node.</summary>
		/// <returns> The potential table.
		/// </returns>
		virtual public Potential Table
		{
			get
			{
				return table;
			}
			
		}
		/// <summary> Returns the neighbors of this node in the triangulated graph.</summary>
		/// <returns> A list of neighbors
		/// </returns>
		virtual public NodeList UndirectedEdges
		{
			get
			{
				return this.undirectedEdges;
			}
			
		}
		/// <summary> Returns the color of this node for graphical display.</summary>
		/// <returns> The color.
		/// </returns>
		protected internal abstract System.Drawing.Color Color{get;}
		/// <summary> Returns the subtype of this node as a String value.</summary>
		/// <returns> The subtype.
		/// </returns>
		protected internal abstract System.String Subtype{get;}
		/// <summary> Returns the type (chance, decision or utility) of this node as a String
		/// value.
		/// </summary>
		/// <returns> The type.
		/// </returns>
		public abstract System.String Type{get;}
		
		/// <summary> The parents of the current node</summary>
		protected internal NodeList parents;
		
		/// <summary> The children of the current node</summary>
		protected internal NodeList children;
		
		/// <summary> The name of the current node. Must be unique within the domain.</summary>
		protected internal System.String name;
		
		/// <summary> A descriptive label. Any String value.</summary>
		protected internal System.String label;
		
		/// <summary> The domain that the current node belongs to.</summary>
		protected internal LimidDomain domain;
		
		/// <summary> The geometrical position of the node when showed graphically.</summary>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		protected internal Point2D position;
		
		/// <summary> The potential table of the node.</summary>
		protected internal Potential table;
		
		/// <summary> The list of neigbors in a moralized or triangulated graph</summary>
		protected internal NodeList undirectedEdges;
		
		/// <summary> The set of directed edges (used for graphical display).</summary>
		protected internal EdgeSet directed;
		
		/// <summary> A list of edge types (whether they are part of the specification, the
		/// moralization or the triangulation). The types are used for displaying
		/// colors depending on type.
		/// </summary>
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		protected internal System.Collections.ArrayList edgeTypes;
		
		/// <summary> Used internally - in particular in the triangulation process.</summary>
		protected internal int triangulationNumber;
		
		/// <summary> Creates a node within a specified domain model.</summary>
		/// <param name="ld">The domain model where the node is to be creatated.
		/// @throws MLPDuplicateNodeNameException The new node is automatically given
		/// a default name. Since names must be unique within domain model, it may
		/// happen that the user has already manually given the default name of this
		/// node to an other node. In that case, an exception is thrown.
		/// </param>
		protected internal Node(LimidDomain ld)
		{
			setDefaultName();
			domain = ld;
			parents = new NodeList();
			children = new NodeList();
			undirectedEdges = new NodeList();
			domain.addNode(this);
			setPosition(new System.Drawing.Point(40, 40));
		}
		
		/// <summary> Copies the properties of an other node.</summary>
		/// <param name="otherNode">The other node.
		/// </param>
		public virtual void  copyNode(Node otherNode)
		{
			int copyNumber = 0;
			bool used = true;
			while (used)
			{
				try
				{
					this.setName(otherNode.name + "_c" + copyNumber);
					used = false;
				}
				catch (MLPDuplicateNodeNameException ex)
				{
					copyNumber++;
				}
			}
			label = otherNode.getLabel();
			domain = otherNode.HomeDomain;
			parents = new NodeList(otherNode.Parents);
			children = new NodeList(otherNode.Children);
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D p = otherNode.getPosition();
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			setPosition(new System.Drawing.Point((int) (p.getX() + 20), (int) (p.getY() + 20)));
		}
		
		/// <summary> Returns the unique name of this node.</summary>
		/// <returns> The name.
		/// </returns>
		public System.String getName()
		{
			return name;
		}
		
		/// <summary> Sets the name of this node.</summary>
		/// <param name="newName">The desired new name.
		/// @throws MLPDuplicateNodeNameException If the name already exists in the
		/// domain.
		/// </param>
		public virtual void  setName(System.String newName)
		{
			domain.renameNode(this.getName(), newName);
		}
		
		/// <summary> Defines the geometric position of this node.</summary>
		/// <param name="pos">The desired position.
		/// </param>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual void  setPosition(Point2D pos)
		{
			position = pos;
			directed = new EdgeSet(this, false);
			// Children must re-calculate edges
			System.Collections.IEnumerator ite = children.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				((Node) ite.Current).reDoEdges();
			}
		}
		
		/// <summary> Recalculates the edges of this node. Usually it is not necessary to call
		/// the method directly. It should only be used by GUI.
		/// </summary>
		public virtual void  reDoEdges()
		{
			directed = new EdgeSet(this, false);
		}
		
		/// <summary> Returns the position of this node.</summary>
		/// <returns> The position.
		/// </returns>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public Point2D getPosition()
		{
			return this.position;
		}
		
		
		/// <summary> Sets the label of the node.</summary>
		/// <param name="newLabel">The desired label.
		/// </param>
		public virtual void  setLabel(System.String newLabel)
		{
			label = newLabel;
		}
		
		/// <summary> Returns the descriptive label.</summary>
		/// <returns> The label.
		/// </returns>
		public System.String getLabel()
		{
			return label;
		}
		
		/// <summary> Returns a string representation of this node. If a label has been defined,
		/// it is returned. Otherwise, the name is returned.
		/// </summary>
		/// <returns> A string representation.
		/// </returns>
		public override System.String ToString()
		{
			if ((System.Object) label == null)
				return name;
			else
				return label;
		}
		
		/// <summary> Adds a child to this node. Should never be called directly. It is used
		/// by the API.
		/// </summary>
		/// <param name="child">The child to be added.
		/// </param>
		protected internal virtual void  addChild(Node child)
		{
			children.Add(child);
		}
		
		/// <summary> Removes a specified child. Should never be called directly. It is used
		/// by the API.
		/// </summary>
		/// <param name="child">The child to be removed.
		/// </param>
		protected internal virtual void  removeChild(Node child)
		{
			children.RemoveElement(child);
		}
		
		/// <summary> Removes all children. Should never be called directly. It is used
		/// by the API.
		/// </summary>
		public virtual void  removeAllChildren()
		{
			children = new NodeList();
		}
		
		/// <summary> Adds a directed edge to the model by defining an other node as being parent
		/// of the present node. The validity of the edge is checked. If violated, an
		/// exception is thrown.
		/// </summary>
		/// <param name="parent">The node to become parent of this node
		/// @throws MLPOperationNotAllowedException Depending on the kind of validity
		/// error, a subtype of this exception is thrown.
		/// </param>
		public virtual void  addParent(Node parent)
		{
			// Edges to the node itself are not allowed
			if (parent == this)
				throw new MLPParentAndChildIdenticalException();
			// The same parent may not be added twice
			if (parents.Contains(parent))
				throw new MLPParentAlreadyDefinedException(parent, this);
			// Utility nodes can not have children
			if (parent is TimeUtilityNode)
				throw new MLPUtilityParentNotAllowedException();
			// The net must be acyclic - ensure that a cycle is never produced
			if (parent.hasAncestor(this))
				throw new MLPCyclicNetException(parent, this);
			// The parent must be a node in the same LIMID
			if (parent.HomeDomain != this.HomeDomain)
				throw new MLPNodeNotMemberException("The parent is does not belong to " + "the same domain as the child");
			parents.Add(parent);
			parent.addChild(this);
			createTable();
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			directed.addEdge(new DirectedEdge(parent, this, HomeDomain.getNodeSize().getWidth() / 2));
			// If an edge is added, the LIMID must be de-compiled
			HomeDomain.deCompile();
			HomeDomain.deMoralize();
		}
		
		/// <summary> Removes a specified parent from this node.</summary>
		/// <param name="parent">The parent to be removed.
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  removeParent(Node parent)
		{
			parents.RemoveElement(parent);
			parent.removeChild(this);
			createTable();
			directed.removeEdgeFrom(parent);
			// If an edge is removed, the LIMID must be de-compiled
			HomeDomain.deCompile();
			HomeDomain.deMoralize();
		}
		
		/// <summary> Removes all parents from this node.
		/// @throws MLPOperationNotAllowedException
		/// </summary>
		public virtual void  removeAllParents()
		{
			NodeList parentList = new NodeList(parents);
			System.Collections.IEnumerator ite = parentList.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node parent = (Node) ite.Current;
				removeParent(parent);
			}
		}
		
		public virtual void  draw(System.Drawing.Graphics graphics)
		{
			SupportClass.GraphicsManager.manager.SetColor(graphics, this.Color);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.FillEllipse(SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) this.getPosition().getX(), (int) this.getPosition().getY(), (int) this.HomeDomain.getNodeSize().getWidth(), (int) this.HomeDomain.getNodeSize().getHeight());
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.DrawEllipse(SupportClass.GraphicsManager.manager.GetPen(graphics), (int) this.getPosition().getX(), (int) this.getPosition().getY(), (int) this.HomeDomain.getNodeSize().getWidth(), (int) this.HomeDomain.getNodeSize().getHeight());
			EdgeSet edges;
			if (LimidDomain.showMoral)
				edges = new EdgeSet(this, LimidDomain.showMoral);
			else
				edges = directed;
			edges.draw(graphics);
			//UPGRADE_TODO: Class 'java.awt.geom.Rectangle2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.FontMetrics.getStringBounds' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			Rectangle2D rect = SupportClass.GraphicsManager.manager.GetFont(graphics).getStringBounds(this.ToString(), graphics);
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double width = rect.getWidth();
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double height = rect.getHeight();
			//UPGRADE_TODO: Method 'java.awt.Graphics.drawString' was converted to 'System.Drawing.Graphics.DrawString' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javaawtGraphicsdrawString_javalangString_int_int"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.awt.FontMetrics.getMaxDescent' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			graphics.DrawString(this.ToString(), SupportClass.GraphicsManager.manager.GetFont(graphics), SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) (this.getPosition().getX() + this.HomeDomain.getNodeSize().getWidth() / 2 - width / 2), (int) (this.getPosition().getY() + this.HomeDomain.getNodeSize().getHeight() / 2 + height / 2 - SupportClass.GetDescent(SupportClass.GraphicsManager.manager.GetFont(graphics))) - SupportClass.GraphicsManager.manager.GetFont(graphics).GetHeight());
		}
		
		public virtual void  drawSelected(System.Drawing.Graphics graphics)
		{
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.FillEllipse(SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) this.getPosition().getX(), (int) this.getPosition().getY(), (int) this.HomeDomain.getNodeSize().getWidth(), (int) this.HomeDomain.getNodeSize().getHeight());
			SupportClass.GraphicsManager.manager.SetColor(graphics, this.Color);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			graphics.FillEllipse(SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) this.getPosition().getX() + 3, (int) this.getPosition().getY() + 3, (int) this.HomeDomain.getNodeSize().getWidth() - 6, (int) this.HomeDomain.getNodeSize().getHeight() - 6);
			SupportClass.GraphicsManager.manager.SetColor(graphics, System.Drawing.Color.Black);
			//UPGRADE_TODO: Class 'java.awt.geom.Rectangle2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.FontMetrics.getStringBounds' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			Rectangle2D rect = SupportClass.GraphicsManager.manager.GetFont(graphics).getStringBounds(this.ToString(), graphics);
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double width = rect.getWidth();
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double height = rect.getHeight();
			//UPGRADE_TODO: Method 'java.awt.Graphics.drawString' was converted to 'System.Drawing.Graphics.DrawString' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javaawtGraphicsdrawString_javalangString_int_int"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.Dimension.getHeight' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: The equivalent in .NET for method 'java.awt.FontMetrics.getMaxDescent' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			graphics.DrawString(this.ToString(), SupportClass.GraphicsManager.manager.GetFont(graphics), SupportClass.GraphicsManager.manager.GetBrush(graphics), (int) (this.getPosition().getX() + this.HomeDomain.getNodeSize().getWidth() / 2 - width / 2), (int) (this.getPosition().getY() + this.HomeDomain.getNodeSize().getHeight() / 2 + height / 2 - SupportClass.GetDescent(SupportClass.GraphicsManager.manager.GetFont(graphics))) - SupportClass.GraphicsManager.manager.GetFont(graphics).GetHeight());
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Drawable getSelectedItem(Point2D point)
		{
			if (this.isPointWithinShape(point))
				return this;
			Drawable selected = directed.getSelectedItem(point);
			if (selected != null)
				return selected;
			return null;
		}
		
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual bool isPointWithinShape(Point2D point)
		{
			// We investigate the distance to the node center
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D pos = this.getPosition();
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double radius = this.HomeDomain.getNodeSize().getWidth() / 2;
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D center = new System.Drawing.Point((int) (pos.getX() + radius), (int) (pos.getY() + radius));
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			return (System.Math.Sqrt(System.Math.Pow((center.getX() - point.getX()), 2.0) + System.Math.Pow((center.getY() - point.getY()), 2.0)) <= radius);
		}
		
		/// <summary> For internal use.</summary>
		internal virtual void  createFirstUndirectedEdges()
		{
			this.undirectedEdges = new NodeList(this.Parents);
			this.undirectedEdges.AddAll(this.children);
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			this.edgeTypes = new System.Collections.ArrayList();
			for (int i = 0, stop = undirectedEdges.Count; i < stop; i++)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				edgeTypes.Add("o");
			}
		}
		
		/// <summary> For internal use.</summary>
		/// <param name="nodes">A list of nodes.
		/// </param>
		/// <returns> true if this node is simplicial within the specified list.
		/// </returns>
		internal virtual bool isSimplicialWithin(NodeList nodes)
		{
			NodeList originals = undirectedEdges;
			NodeList neighbors = new NodeList(undirectedEdges);
			// Prune for already eliminated nodes
			System.Collections.IEnumerator ite = originals.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				DiscreteNode dNode = (DiscreteNode) ite.Current;
				if (!nodes.Contains(dNode))
					neighbors.RemoveElement(dNode);
			}
			DiscreteNode[] nodeSet = new DiscreteNode[neighbors.Count];
			nodeSet = (DiscreteNode[]) neighbors.ToArray(nodeSet);
			for (int i = 0; i < nodeSet.Length; i++)
				for (int j = 0; j < nodeSet.Length; j++)
					if (i != j)
					{
						if (!nodeSet[i].isConnectedTo(nodeSet[j]))
							return false;
					}
			return true;
		}
		
		/// <summary> For internal use. Adds an undirected edge.</summary>
		/// <param name="target">The node to connect to.
		/// </param>
		/// <param name="type">The type of edge.
		/// </param>
		internal virtual void  addUndirectedEdge(Node target, System.String type)
		{
			this.undirectedEdges.Add(target);
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			this.edgeTypes.Add(type);
		}
		
		/// <summary> For internal use. Removes all undirected edges.</summary>
		internal virtual void  removeUndirectedEdges()
		{
			this.undirectedEdges = new NodeList();
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			this.edgeTypes = new System.Collections.ArrayList();
		}
		
		/// <summary> For internal use. Removes a specified undirected edge.</summary>
		/// <param name="node">The node to which the edge should be deleted.
		/// </param>
		internal virtual void  removeUndirectedEdge(Node node)
		{
			int i = undirectedEdges.IndexOf(node);
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.remove' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			edgeTypes.RemoveAt(i);
			undirectedEdges.RemoveElement(node);
		}
		
		/// <summary> For internal use.</summary>
		/// <param name="node">An other node
		/// </param>
		/// <returns> true if there is an undirected edge to the other node.
		/// </returns>
		internal virtual bool isConnectedTo(Node node)
		{
			return undirectedEdges.Contains(node);
		}
		
		/// <summary> Writes the potential table of this node to an output stream. The
		/// format is in accordance with a Hugin NET file.
		/// </summary>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public virtual void  potentialToNet(System.IO.StreamWriter stream)
		{
			stream.Write("potential (" + this.getName());
			if (this.parents.Count > 0)
				stream.Write(" |");
			System.Collections.IEnumerator ite = parents.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				stream.Write(" " + node.getName());
			}
			stream.Write(")\n");
			stream.Write("{\n");
			stream.Write("    data\n");
			stream.Write("     = ");
			Potential table = this.Table;
			NodeList tableNodes = table.Nodes;
			int[] oldState = new int[tableNodes.Count];
			int[] maximums = new int[tableNodes.Count];
			for (int i = 0; i < oldState.Length; i++)
			{
				oldState[i] = - 1;
				maximums[i] = ((DiscreteNode) tableNodes[i]).getNumberOfStates();
			}
			table.initialize();
			while (table.hasNextState())
			{
				int[] state = table.nextState();
				if (state[state.Length - 1] == 0)
				{
					stream.Write("(");
					int j = state.Length - 2;
					while (j > - 1 && state[j] == 0)
					{
						stream.Write("(");
						j--;
					}
				}
				stream.Write(" " + table.getDataItem(state));
				if (state[state.Length - 1] == maximums[state.Length - 1] - 1)
				{
					stream.Write(")");
					int j = state.Length - 2;
					while (j > - 1 && state[j] == maximums[j] - 1)
					{
						stream.Write(")");
						j--;
					}
					if (j > - 1)
						stream.Write("\n        ");
				}
			}
			stream.Write(";\n}\n\n");
		}
		
		/// <summary> Returns the geometric center of this node.</summary>
		/// <returns> The center.
		/// </returns>
		//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
		public virtual Point2D findCenter()
		{
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D pos = this.getPosition();
			//UPGRADE_TODO: Method 'java.awt.Dimension.getWidth' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			double radius = this.HomeDomain.getNodeSize().getWidth() / 2;
			//UPGRADE_TODO: Class 'java.awt.geom.Point2D' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			Point2D center = new System.Drawing.Point((int) (pos.getX() + radius), (int) (pos.getY() + radius));
			return center;
		}
		
		/// <summary> Checks whether this node has an other node among its descendants (at any
		/// level). Used for validation of new edges.
		/// </summary>
		/// <param name="node">The node to be search for among the decendants of this node.
		/// </param>
		/// <returns> true if node is descendant of this node.
		/// </returns>
		public virtual bool hasDescendant(Node node)
		{
			return node.hasAncestor(this);
		}
		
		/// <summary> Checks whether this node has an other node among its ancestors (at any
		/// level). Used for validation of new edges.
		/// </summary>
		/// <param name="node">The node to be search for among the ancestors of this node.
		/// </param>
		/// <returns> true if node is ancestor of this node.
		/// </returns>
		public virtual bool hasAncestor(Node node)
		{
			if (parents.Contains(node))
				return true;
			else
			{
				System.Collections.IEnumerator ite = parents.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node parent = (Node) ite.Current;
					if (parent.hasAncestor(node))
						return true;
				}
			}
			return false;
		}
		
		/// <summary> For internal use by the domain to check whether the graph is connected</summary>
		/// <param name="reached">The list of nodes already checked.
		/// </param>
		internal virtual void  reachNode(NodeList reached)
		{
			if (triangulationNumber == - 1)
			{
				reached.Add(this);
				triangulationNumber = 0;
				System.Collections.IEnumerator ite = parents.GetEnumerator();
				if (!(this is TimeUtilityNode))
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (ite.MoveNext())
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						Node parent = (Node) ite.Current;
						parent.reachNode(reached);
					}
				}
				ite = children.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node child = (Node) ite.Current;
					child.reachNode(reached);
				}
			}
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public virtual SupportClass.CollectionSupport addSelectedItems(System.Drawing.Rectangle r, SupportClass.CollectionSupport allready)
		{
			//UPGRADE_TODO: Method 'java.awt.geom.RectangularShape.contains' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			if (r.contains(this.findCenter()))
			{
				allready.Add(this);
			}
			allready = directed.addSelectedItems(r, allready);
			return allready;
		}
		
		/// <summary> Returns true if this node has values associated with its states. The default
		/// is false.
		/// </summary>
		/// <returns> true if this node supports state values
		/// </returns>
		public virtual bool hasStateValues()
		{
			return false;
		}
		
		/// <summary> Writes the basic properties of this node to an output stream. The format is
		/// in accordance with the default XML syntax for Bayesian Networks.
		/// </summary>
		/// <param name="blanks">A string of blanks for indentation.
		/// </param>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public virtual void  toStream(System.String blanks, System.IO.StreamWriter stream)
		{
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			stream.Write(blanks + "<VAR NAME=\"" + getName() + "\" TYPE = \"discrete\" XPOS=\"" + ((int) getPosition().getX() * 100) + "\"" + " YPOS=\"" + ((int) getPosition().getY() * 100) + "\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<DESCRIPTION>" + getLabel() + "</DESCRIPTION>\n");
			stream.Write(blanks + "<PROPERTY NAME = \"nodetype\">\n");
			blanks = System.String.Concat(blanks, "  ");
			stream.Write(blanks + "<PROPVALUE>" + Type + "</PROPVALUE>\n");
			blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
			stream.Write(blanks + "</PROPERTY>\n");
			if (Subtype.Length > 0)
			{
				stream.Write(blanks + "<PROPERTY NAME = \"subtype\">\n");
				blanks = System.String.Concat(blanks, "  ");
				stream.Write(blanks + "<PROPVALUE>" + Subtype + "</PROPVALUE>\n");
				blanks = blanks.Substring(0, (blanks.Length - 2) - (0));
				stream.Write(blanks + "</PROPERTY>\n");
			}
		}
		
		/// <summary> This method creates an initial table for a node before any parents have been
		/// added. It is called by the constructors of the various sub-classes. A call
		/// to createTable() would have the same effect, but this method does NOT throw
		/// the MLPOperationNotAllowedException, since there cannot be an illegal
		/// parent in a node that has no parents at all. This method should never be
		/// called directly.
		/// </summary>
		protected internal virtual void  createInitialTable()
		{
			try
			{
				createTable();
			}
			catch (MLPOperationNotAllowedException ex)
			{
				throw new System.SystemException("An unexpected error occured when an initial " + "table of a discrete node was created");
			}
		}
		
		/// <summary> For internal use. Defines a default name.</summary>
		protected internal abstract void  setDefaultName();
		
		/// <summary> Create an empty table for this node. Used by the API.
		/// @throws MLPOperationNotAllowedException
		/// </summary>
		public abstract void  createTable();
		
		/// <summary> Writes this node in Hugin NET format to an output stream.</summary>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public abstract void  toNet(System.IO.StreamWriter stream);
		
		/// <summary> Clone this node</summary>
		/// <returns> A clone of this node.
		/// @throws java.lang.CloneNotSupportedException
		/// </returns>
		//UPGRADE_TODO: The equivalent of method 'java.lang.Object.clone' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		protected internal abstract System.Object Clone();
	}
}