using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> A class representing a list of nodes. It is basically a LinkedList with a few
	/// additional facilities. In general, any attempt to add objects that are not
	/// nodes will result in a runtime exception of type IllegalArgumentException
	/// being thrown. That behavior is the only reason for overriding the "add methods"
	/// of class LinkedList.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class NodeList:SupportClass.ListCollectionSupport
	{
		
		/// <summary> Creates a new empty node list.</summary>
		public NodeList()
		{
		}
		
		/// <summary> Creates a node list containing the elements of the collection. If the
		/// collection contains elements that are not nodes, a runtime exception
		/// is thrown
		/// </summary>
		/// <param name="c">A collection of nodes.
		/// </param>
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public NodeList(SupportClass.CollectionSupport c):this()
		{
			System.Collections.IEnumerator ite = c.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				System.Object o = ite.Current;
				if (o is Node)
					this.Add(o);
				else
					throw new System.ArgumentException("The input collection contains at least one element which is NOT a node");
			}
		}
		
		/// <summary> Returns a string representation of this node list</summary>
		/// <returns> A string representation of this node list
		/// </returns>
		public override System.String ToString()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			System.Collections.IEnumerator ite = this.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				//UPGRADE_TODO: The equivalent in .NET for method 'java.lang.Object.toString' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				sb.Append(node.ToString());
				if (node != this.GetLast())
					sb.Append(", ");
			}
			return sb.ToString();
		}
		
		/// <summary> Checks whether this node list contains a simplicial node if supplemented
		/// with the specified additional nodes.
		/// </summary>
		/// <param name="supplement">The additional nodes.
		/// </param>
		/// <returns> true if the list contains a simplicial node.
		/// </returns>
		internal virtual bool hasSimplicialNode(NodeList supplement)
		{
			NodeList extended = new NodeList(this);
			extended.AddAll(supplement);
			System.Collections.IEnumerator ite = this.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				if (node.isSimplicialWithin(supplement))
					return true;
			}
			return false;
		}
		
		/// <summary> Copies the node list and return a list of clones. Used by the "Copy"
		/// facility of the GUI.
		/// </summary>
		/// <returns> A list of clones
		/// @throws CloneNotSupportedException
		/// </returns>
		public virtual NodeList copy()
		{
			NodeList newNodes = new NodeList();
			// Clone the nodes
			System.Collections.IEnumerator ite = this.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				System.Object node = ((Node) ite.Current).MemberwiseClone();
				newNodes.Add(node);
			}
			return newNodes;
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.util.LinkedList.addFirst' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		public void  addFirst(System.Object parm1)
		{
			if (parm1 is Node)
				base.Insert(0, parm1);
			else
				throw new System.ArgumentException("The input parameter is NOT a node");
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.util.LinkedList.addLast' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		public void  addLast(System.Object parm1)
		{
			if (parm1 is Node)
			{
				//UPGRADE_TODO: The equivalent in .NET for method 'java.util.LinkedList.addLast' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				base.Add(parm1);
			}
			else
				throw new System.ArgumentException("The input parameter is NOT a node");
		}
		
		public override bool Add(System.Object parm1)
		{
			if (parm1 is Node)
				return base.Add(parm1);
			else
				throw new System.ArgumentException("The input parameter is NOT a node");
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public override bool AddAll(SupportClass.CollectionSupport parm1)
		{
			System.Collections.IEnumerator ite = parm1.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				System.Object o = ite.Current;
				Add(o);
			}
			return true;
		}
		
		public override void Insert(int parm1, System.Object parm2)
		{
			if (parm2 is Node)
				base.Insert(parm1, parm2);
			else
				throw new System.ArgumentException("The input parameter is NOT a node");
		}
		
		//UPGRADE_ISSUE: Class hierarchy differences between ''java.util.Collection'' and ''SupportClass.CollectionSupport'' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		public override bool AddAll(int parm1, SupportClass.CollectionSupport parm2)
		{
			System.Collections.IEnumerator ite = parm2.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				System.Object o = ite.Current;
				if (!(o is Node))
					throw new System.ArgumentException("The input collection contains at least one element which is NOT a node");
			}
			return base.AddAll(parm1, parm2);
		}
		
		/// <summary> Moves these nodes to an other specified Limid. Edges to nodes outside the
		/// list may optionally be deleted. Used by the "Cut" facility of the GUI.
		/// </summary>
		/// <param name="otherLimid">The destination Limid for the nodes.
		/// </param>
		/// <param name="pruneExternals">Determines whether external edges are removed.
		/// @throws MLPDuplicateNodeNameException
		/// @throws MLPOperationNotAllowedException
		/// </param>
		public virtual void  moveToLimid(LimidDomain otherLimid, bool pruneExternals)
		{
			System.Collections.IEnumerator ite = this.GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				Node node = (Node) ite.Current;
				node.Limid = otherLimid;
			}
			// Prune links to nodes outside new domain
			if (pruneExternals)
			{
				ite = this.GetEnumerator();
				//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
				while (ite.MoveNext())
				{
					//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
					Node node = (Node) ite.Current;
					// Prune external parents
					NodeList parentList = new NodeList(node.Parents);
					System.Collections.IEnumerator jte = parentList.GetEnumerator();
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (jte.MoveNext())
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						Node parent = (Node) jte.Current;
						if (parent.HomeDomain != otherLimid)
							node.removeParent(parent);
					}
					// Prune external children
					NodeList childList = new NodeList(node.Children);
					jte = childList.GetEnumerator();
					//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
					while (jte.MoveNext())
					{
						//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
						Node child = (Node) jte.Current;
						if (child.HomeDomain != otherLimid)
							child.removeParent(node);
					}
				}
			}
		}
		
		/// <summary> Returns the intersection of this node list and an other node list. The
		/// intersection is a list of the nodes that are members of <i>both</i> lists.
		/// The method is used by the test methods. Since it is inefficiently coded
		/// it should not be used where complexity matters.
		/// </summary>
		/// <param name="otherList">The other node list.
		/// </param>
		/// <returns> The intersection.
		/// </returns>
		public virtual NodeList getIntersectionWith(NodeList otherList)
		{
			NodeList intersection = new NodeList();
			System.Collections.IEnumerator ite = GetEnumerator();
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (ite.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				System.Object node = ite.Current;
				if (otherList.Contains(node))
					intersection.Add(node);
			}
			return intersection;
		}
	}
}