using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The Potential interface defines all the methods needed in order to represent
	/// the data of a potential table.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	
	public interface Potential
		{
			//UPGRADE_NOTE: Respective javadoc comments were merged.  It should be changed in order to comply with .NET documentation conventions.
			/// <summary> Returns the data as a one-dimensional array.</summary>
			/// <returns> An array of potential values.
			/// </returns>
			/// <summary> Specifies the data of the table as a one dimensional array.</summary>
			/// <param name="parm1">The array of potential values.
			/// </param>
			double[] Data
			{
				get;
				
				set;
				
			}
			/// <summary> Sets all entries to the value v.</summary>
			/// <param name="v">The desired value.
			/// </param>
			int IdenticalValues
			{
				set;
				
			}
			/// <summary> Returns the number of parent configurations for this table.</summary>
			/// <returns> The number of parent configurations.
			/// </returns>
			int ParentStateCount
			{
				get;
				
			}
			/// <summary> Returns the number of states of the node that owns this table.</summary>
			/// <returns> The number of states.
			/// </returns>
			int StateCount
			{
				get;
				
			}
			/// <summary> Returns the list of nodes defining the configurations of this table.</summary>
			/// <returns> A list of nodes.
			/// </returns>
			NodeList Nodes
			{
				get;
				
			}
			/// <summary> Returns the number of entries in this table.</summary>
			/// <returns> The number of entries.
			/// </returns>
			int Size
			{
				get;
				
			}
			
			/// <summary> Returns a specified entry of the potential table.</summary>
			/// <param name="parm1">The index of the desired entry.
			/// </param>
			/// <returns> The value of the entry.
			/// </returns>
			double getDataItem(int parm1);
			
			/// <summary> Returns a specified entry of the potential table. Calling this method is
			/// equivalent to the call getDataItem(convertToIndex(state)).
			/// </summary>
			/// <param name="state">An array of state specifications of the nodes of the table.
			/// </param>
			/// <returns> The value of the entry.
			/// </returns>
			double getDataItem(int[] state);
			
			/// <summary> Specifies a specific entry of the potential table.</summary>
			/// <param name="index">The index of the entry.
			/// </param>
			/// <param name="value">The value of the entry.
			/// </param>
			void  setDataItem(int index, double value_Renamed);
			
			/// <summary> Converts an array of node states to an index of the potential.</summary>
			/// <param name="state">The array of node states.
			/// </param>
			/// <returns> The resulting index in the table.
			/// </returns>
			int convertToIndex(int[] state);
			
			/// <summary> Returns an array of node states given an index of the set of parent
			/// configurations.
			/// </summary>
			/// <param name="index">The index of the set of parent configurations.
			/// </param>
			/// <returns> The array of node states corresponding to the index.
			/// </returns>
			int[] convertParentIndexToStateArray(int index);
			
			/// <summary> Specifies a specific entry of the potential table. A call of this method
			/// ids equivalent to the call setDataItem(convertToIndex(state), value).
			/// </summary>
			/// <param name="state">An array of node states.
			/// </param>
			/// <param name="value">The value of the entry.
			/// </param>
			void  setDataItem(int[] state, double value_Renamed);
			
			/// <summary> It is possible to iterate over node configurations of this table. In orter
			/// to reset the iterator, the initialize() method must be called. It causes
			/// the table pointer to point to the first element.
			/// </summary>
			void  initialize();
			
			/// <summary> Part of the iterator facility. Returns true, if more configuations are
			/// available. A call to hasNextState() increments the table pointer!!!
			/// </summary>
			/// <returns> true if more configurations are available.
			/// </returns>
			bool hasNextState();
			
			/// <summary> Part of the iterator facility. Returns the next configuration as an array
			/// of node states. The method does NOT increment the table pointer!!!
			/// </summary>
			/// <returns> The next configuration as an array of node states.
			/// </returns>
			int[] nextState();
			
			/// <summary> Multiplies the content of this table with the content of an other table.
			/// The multiplication is element by element.
			/// </summary>
			/// <param name="ut">An other potential
			/// </param>
			/// <returns> The contraction of this table with the table ut.
			/// @throws MLPOperationNotAllowedException
			/// </returns>
			PotentialTable contractWith(PotentialTable ut);
			
			/// <summary> Returns a text-oriented tabular print of this table combined with an other
			/// table in the same domain. No check of domain is performed, since the method
			/// is mostly thought of as being for debugging purposes.
			/// </summary>
			/// <param name="otherTable">The other table. If null, only this table is printed.
			/// </param>
			/// <returns> A string (with line breaks) containing the table
			/// </returns>
			System.String toString(Potential otherTable);
		}
}