using System;
namespace dk.esthauge.limid.api
{
	
	/// <summary> The class representing a utility node.
	/// <p>Title: Limited Memory Influence Diagrams</p>
	/// <p>Description: Implementation and GUI</p>
	/// <p>Copyright: Copyright (c) 2003</p>
	/// </summary>
	/// <author>  Anders Ringgaard Kristensen
	/// </author>
	/// <version>  1.0
	/// </version>
	public class TimeUtilityNode:Node
	{
		/// <summary> Returns the color of a utility node. This default implementation uses
		/// the static field LimidDomain.utilityNodeColor for representation of
		/// this color. Subclasses may override the method for more sofisticated
		/// coloring.
		/// </summary>
		/// <returns> The color.
		/// </returns>
		/// <seealso cref="LimidDomain#utilityNodeColor">
		/// </seealso>
		override protected internal System.Drawing.Color Color
		{
			get
			{
				return this.HomeDomain.utilityNodeColor;
			}
			
		}
		/// <summary> Returns the time table.</summary>
		/// <returns> The time table.
		/// </returns>
		virtual public Potential TimeTable
		{
			get
			{
				return (Potential) itemTables[1];
			}
			
		}
		/// <summary> Returns the empty string. No subtypes are defined for utility nodes.</summary>
		/// <returns> The empty string.
		/// </returns>
		override protected internal System.String Subtype
		{
			get
			{
				return "";
			}
			
		}
		/// <summary> Returns the string "utility"</summary>
		/// <returns> The string "Utility"
		/// </returns>
		override public System.String Type
		{
			get
			{
				return "utility";
			}
			
		}
		/// <summary> Returns all potential tables in a list.</summary>
		/// <returns> A list of all potential tables.
		/// </returns>
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		virtual public System.Collections.ArrayList AllPotentials
		{
			get
			{
				return this.itemTables;
			}
			
		}
		
		/// <summary> Used for creating unique default names</summary>
		internal static int counter = 0;
		
		/// <summary> A list of potential tables. Mainly for future extensions. At present it
		/// contains two tables: A utility table and a time table. The time table
		/// is not yet used.
		/// </summary>
		//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
		protected internal System.Collections.ArrayList itemTables;
		
		/// <summary> Creates a new utility node within the specified model.</summary>
		/// <param name="ld">The model.
		/// @throws MLPDuplicateNodeNameException
		/// </param>
		public TimeUtilityNode(LimidDomain ld):base(ld)
		{
			//UPGRADE_ISSUE: Class hierarchy differences between 'java.util.ArrayList' and 'System.Collections.ArrayList' may cause compilation errors. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1186"'
			itemTables = new System.Collections.ArrayList();
			// It must hold at least 2 tables
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			itemTables.Add(table);
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.add' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			itemTables.Add(table);
			createInitialTable();
			counter++;
		}
		
		/// <summary> Creates and initializes the list of tables.
		/// @throws MLPOperationNotAllowedException
		/// </summary>
		public override void  createTable()
		{
			NodeList nodes = new NodeList(this.Parents);
			//UPGRADE_TODO: The equivalent in .NET for method 'java.util.ArrayList.size' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
			for (int i = 0, stop = itemTables.Count; i < stop; i++)
			{
				PotentialTable tab = new PotentialTable(nodes);
				tab.IdenticalValues = 0;
				System.Object item;
				item = itemTables[i];
				itemTables[i] = tab;
				System.Object generatedAux = item;
			}
			this.table = (Potential) itemTables[0];
		}
		
		/// <summary> The defaultname is the letter "D" followed by a number.</summary>
		protected internal override void  setDefaultName()
		{
			name = "U" + counter;
		}
		
		/// <summary> Returns a specified table in the table list.</summary>
		/// <param name="i">The index of the table.
		/// </param>
		/// <returns> The table.
		/// </returns>
		public virtual Potential getItemTable(int i)
		{
			return (Potential) itemTables[i];
		}
		
		/// <summary> Writes the basic properties of this node in the format of a Hugin NET file.</summary>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public override void  toNet(System.IO.StreamWriter stream)
		{
			stream.Write("utility " + this.getName() + "\n");
			stream.Write("{\n");
			if ((System.Object) this.label != null)
				stream.Write("    label = \"" + this.getLabel() + "\";\n");
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getX' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			//UPGRADE_TODO: Method 'java.awt.geom.Point2D.getY' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			stream.Write("    position = (" + (int) this.getPosition().getX() + " " + (int) this.getPosition().getY() + ");\n");
			stream.Write("}\n\n");
		}
		
		//UPGRADE_TODO: The equivalent of method 'java.lang.Object.clone' is not an override method. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1143"'
		protected internal System.Object Clone()
		{
			try
			{
				TimeUtilityNode newNode = new TimeUtilityNode(this.HomeDomain);
				newNode.copyNode(this);
				return newNode;
			}
			catch (MLPException ex)
			{
				return null;
			}
		}
		
		/// <summary> Writes basic information about this node in the format of an XML file.</summary>
		/// <param name="blanks">A string of leading blanks.
		/// </param>
		/// <param name="stream">An output stream writer.
		/// @throws IOException
		/// </param>
		public override void  toStream(System.String blanks, System.IO.StreamWriter stream)
		{
			base.toStream(blanks, stream);
			stream.Write(blanks + "</VAR>\n");
		}
	}
}